package com.softslate.commerce.administrator.core;

import java.util.Map;

import javax.servlet.RequestDispatcher;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.administrator.AdministratorProcessor;
import com.softslate.commerce.businessobjects.administrator.UpgradesProcessor;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * @author David Tobey
 */
public class LoginAction extends BaseAction {
	static Log log = LogFactory.getLog(LoginAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form, HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		LoginForm baseForm = (LoginForm) form;
		synchronized (baseForm.getRequest().getSession()) {
			if (baseForm.getUser().getAdministrator() == null) {
				ensureAllAdministratorFieldsExist(baseForm);
				AdministratorProcessor administratorProcessor =  baseForm
						.getBusinessObjectFactory().createObject(AdministratorProcessor.class);
				Map result = administratorProcessor.processLogin(PropertyUtils.describe(baseForm));
				// Process errors
				if (result != null && result.get("resultCode") != null && !result.get("resultCode").equals("0")) {
					String resultCode = (String) result.get("resultCode");
					if (resultCode.equals("errors.passwordHasExpired")) {
						baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(resultCode));
						saveMessages(baseForm.getRequest(), (ActionMessages) baseForm.getMessages());
						return mapping.findForward("passwordHasExpired");
					} else if (!resultCode.equals("errors.userAlreadyLoggedIn")) {
						baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(resultCode));
						saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
						return mapping.findForward("failure");
					}
				}
			}
			String originalRequest = baseForm.getOriginalRequest();
			if (originalRequest == null) {
				originalRequest = (String) request.getAttribute("originalRequest");
			}
			if (originalRequest != null) {
				RequestDispatcher rd = request
						.getRequestDispatcher(originalRequest);
				rd.forward(request, response);
				return null;
			}
			return mapping.findForward("success");
		}
	}

	protected void ensureAllAdministratorFieldsExist(LoginForm baseForm) {

		// Set the current version of the database objects.
		String currentVersion = (String) baseForm.getSettings().getValue("dataObjectsVersion");
		if (currentVersion != null && currentVersion.compareTo("3.3.1") < 0) {
			try {
				UpgradesProcessor upgradesProcessor = (UpgradesProcessor) baseForm.getBusinessObjectFactory()
						.createObjectFromClassName(
								"com.softslate.commerce.businessobjects.administrator.UpgradesProcessor");
				String dbType = baseForm.getBusinessObjectFactory().getDaoFactory().getDatabaseType().toLowerCase();
				String pathToUpgrades = getServlet().getServletContext().getRealPath(
						"/WEB-INF/classes/resources/upgrades/");
				String pathToScript = pathToUpgrades + System.getProperty("file.separator")
						+ "pre-upgrade_3.3.0_3.3.1_" + dbType + ".sql";
				log.info("Adding Administrator fields using: " + pathToScript);
				upgradesProcessor.processSingleSqlScript(pathToScript);
			} catch (Exception e) {
				log.error("Exception adding new administrator fields: ", e);
			}
		}
	}
}
