package com.softslate.commerce.administrator.core;

import java.util.Arrays;
import java.util.Collection;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.customer.core.AppMessageResources;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * @author David Tobey
 * 
 */
public class StoreMessagesFormAction extends BaseAction {

	static Log log = LogFactory.getLog(StoreMessagesFormAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		StoreMessagesForm baseForm = (StoreMessagesForm) form;
		baseForm.initializeProperties(mapping, request);

		// Load the given bundle into the request scope for display by the JSP.
		try {
			AppMessageResources resources = (AppMessageResources) getServlet()
					.getServletContext().getAttribute(
							"/" + baseForm.getBundle());
			resources.getMessage("initialize");
			resources.getCustomMessageResources().getMessage("initialize");
			Map messages = resources.getMessages();
			Map customMessages = resources.getCustomMessageResources().getMessages();
			messages.putAll(customMessages);
			request.setAttribute("messages", resources.getMessages());
			if (log.isDebugEnabled()) {
				log.debug("Bundle: " + baseForm.getBundle());
				log.debug("Found messages to edit: " + messages);
				log.debug("Custom messages: " + customMessages);
			}
		} catch (Exception e) {
			baseForm.getErrors()
					.add(
							ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("errors.settingsLoadError", e
									.toString()));
			saveErrors(request, (ActionMessages) baseForm.getErrors());
			return mapping.findForward("error");
		}

		// Create breadcrumbing collection
		String pageTitle = "Store Messages";
		if (baseForm.getBundle() == null) {
			pageTitle = "General Messages";
		} else if (baseForm.getBundle().equals("core")) {
			pageTitle = "General Messages";
		} else if (baseForm.getBundle().equals("product")) {
			pageTitle = "Product Catalog Messages";
		} else if (baseForm.getBundle().equals("customer")) {
			pageTitle = "Customer Account Messages";
		} else if (baseForm.getBundle().equals("order")) {
			pageTitle = "Checkout/Cart Messages";
		} else if (baseForm.getBundle().equals("payment")) {
			pageTitle = "Payment Messages";
		}
		Collection bcs = Arrays
				.asList(new String[] { "Home", pageTitle });
		request.setAttribute("breadcrumbs", bcs);

		if (log.isDebugEnabled())
			log.debug("Finished with Action.");
		return mapping.findForward("success");
	}
}
