package com.softslate.commerce.administrator.core;

import java.util.Arrays;
import java.util.Collection;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;

import com.softslate.commerce.businessobjects.core.Setting;
import com.softslate.commerce.businessobjects.core.Settings;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class corresponding to requests to
 * "/administrator/UserSettingEditForm.do". Loads in the current values of the
 * record being edited and populates a <code>UserSettingAddEditForm</code>
 * object.
 * 
 * @author Jason McWilliams
 * 
 */
public class UserSettingEditFormAction extends BaseAction {

	static Log log = LogFactory.getLog(UserSettingEditFormAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		UserSettingAddEditForm baseForm = (UserSettingAddEditForm) form;
		baseForm.initializeProperties(mapping, request);

		// Get the record to edit
		Settings settingsBean =  baseForm.getBusinessObjectFactory()
				.createObject(Settings.class);
		Setting setting = settingsBean.loadSettingFromID(PropertyUtils
				.describe(form));

		if (!setting.getType().equals("USERDEFINED")) {
			baseForm
					.getMessages()
					.add(
							ActionErrors.GLOBAL_MESSAGE,
							new ActionMessage("errors.detail",
									"Sorry, you can't edit/delete this type of setting from this screen."));
			addErrors(baseForm.getRequest(), baseForm.getMessages());
			return mapping.findForward("failure");
		}

		BeanUtils.copyProperties(baseForm, setting);

		// Enforce role permissions
		if (!request.isUserInRole("superuser")
				&& !request.isUserInRole("administrator")
				&& !request.isUserInRole("settingsEdit") && !request.isUserInRole("settingsCustomEdit")) {
			request.setAttribute("editPermission", "false");
		} else {
			request.setAttribute("editPermission", "true");
		}

		// Create breadcrumbing collection
		Collection bcs = Arrays.asList(new String[] { "Home",
				"Custom Settings", "Custom Setting Details" });
		request.setAttribute("breadcrumbs", bcs);

		request.setAttribute("userSetting", setting);
		request.setAttribute("userSettingAddEditForm", baseForm);
		return mapping.findForward("success");

	}
}
