package com.softslate.commerce.administrator.extender;

import java.util.Properties;

public class Entity {

	private String incomingName = null;
	private String fullyQualifiedName = null;
	private String fullyQualifiedInterfaceName = null;
	private String className = null;
	private String interfaceName = null;
	private String initialLowerCaseClassName = null;
	private String initialLowerCaseInterfaceName = null;
	private String pluralInterfaceName = null;
	private String pluralLowerName = null;
	private String componentsKey = null;
	private String lastPackageName = null;
	private String packageAboveBusinessObjects = null;
	private String packageUnderBusinessObjects = null;
	private String packageUnderEvents = null;
	private String packageUnderDAOs = null;
	private String packageUnderAdministrator = null;
	private String pathToPackageAboveBusinessObjects = null;
	private String pathToPackageUnderBusinessObjects = null;
	private String pathToPackageUnderDAOs = null;
	private String pathToPackageUnderEvents = null;
	private String pathToResourcesSubdirectory = null;
	private String pathToPackageUnderAdministrator = null;
	private String pathToDefaultAdministratorJSPs = null;
	private String pathToCustomAdministratorJSPs = null;
	private String pathToCustomCoreAdministratorJSPs = null;
	private String pathToCustomStrutsConfFile = null;
	private String pathToCustomTilesConfFile = null;
	private Properties appComponents = null;
	private String dbTableName = null;
	private String nameField = null;
	private String cappedNameWithSpaces = null;
	private String cappedNameWithSpacesShortened = null;
	private String lowerNameWithSpaces = null;
	private String cappedPluralNameWithSpaces = null;
	private String lowerPluralNameWithSpaces = null;

	public Entity(String incomingName, Properties appComponents) {
		setIncomingName(incomingName);
		setAppComponents(appComponents);
	}

	public String getFullyQualifiedName() {
		if (fullyQualifiedName == null) {
			fullyQualifiedName = getIncomingName();
			if (!getIncomingName().endsWith("Bean")) {
				fullyQualifiedName = getIncomingName() + "Bean";
			}
		}
		return fullyQualifiedName;
	}

	public String getFullyQualifiedInterfaceName() {
		if (fullyQualifiedInterfaceName == null) {
			fullyQualifiedInterfaceName = getFullyQualifiedName().substring(0, getFullyQualifiedName().length() - 4);
		}
		return fullyQualifiedInterfaceName;
	}

	public String getInterfaceName() {
		if (interfaceName == null) {
			String[] packagesAndClassName = getFullyQualifiedName().split("\\.");
			interfaceName = packagesAndClassName[packagesAndClassName.length - 1];
			if (interfaceName.endsWith("Bean")) {
				interfaceName = interfaceName.substring(0, interfaceName.length() - 4);
			}
		}
		return interfaceName;
	}

	public String getClassName() {
		if (className == null) {
			String[] packagesAndClassName = getFullyQualifiedName().split("\\.");
			className = packagesAndClassName[packagesAndClassName.length - 1];
		}
		return className;
	}

	public String getLowerName() {
		return getInitialLowerCaseInterfaceName();
	}

	public String getCappedName() {
		return getInterfaceName();
	}

	public String getPluralName() {
		return getPluralInterfaceName();
	}

	public String getInitialLowerCaseClassName() {
		if (initialLowerCaseClassName == null) {
			String className = getClassName();
			initialLowerCaseClassName = className.substring(0, 1).toLowerCase() + className.substring(1);
		}
		return initialLowerCaseClassName;
	}

	public String getInitialLowerCaseInterfaceName() {
		if (initialLowerCaseInterfaceName == null) {
			String interfaceName = getInterfaceName();
			initialLowerCaseInterfaceName = interfaceName.substring(0, 1).toLowerCase() + interfaceName.substring(1);
		}
		return initialLowerCaseInterfaceName;
	}

	public String getPluralInterfaceName() {
		if (pluralInterfaceName == null) {
			pluralInterfaceName = pluralize(getInterfaceName());
		}
		return pluralInterfaceName;
	}

	public String getPluralLowerName() {
		if (pluralLowerName == null) {
			pluralLowerName = getPluralInterfaceName().substring(0, 1).toLowerCase()
					+ getPluralInterfaceName().substring(1);
		}
		return pluralLowerName;
	}

	// public String getPluralNameInSentence() {
	// if (pluralNameInSentence == null) {
	// String[] parts = getPluralInterfaceName().split("[A-Z]");
	// pluralNameInSentence = StringUtils.join(parts, " ");
	// pluralNameInSentence = pluralNameInSentence.toLowerCase();
	// }
	// return pluralNameInSentence;
	// }

	public String getComponentsKey() {
		if (componentsKey == null) {
			componentsKey = getInitialLowerCaseInterfaceName() + "Implementer";
		}
		return componentsKey;
	}

	public String getLastPackageName() {
		if (lastPackageName == null) {
			String[] packagesAndClassName = getFullyQualifiedName().split("\\.");
			lastPackageName = packagesAndClassName[packagesAndClassName.length - 2];
		}
		return lastPackageName;
	}

	public String getPackageAboveBusinessObjects() {
		if (packageAboveBusinessObjects == null) {
			String[] packagesAndClassName = getFullyQualifiedName().split("\\.");
			for (int i = 0; i < packagesAndClassName.length - 3; i++) {
				String packageName = packagesAndClassName[i];
				if (i > 0) {
					packageAboveBusinessObjects = packageAboveBusinessObjects + "." + packageName;
				} else {
					packageAboveBusinessObjects = packageName;
				}
			}
		}
		return packageAboveBusinessObjects;
	}

	public String getPackageUnderBusinessObjects() {
		if (packageUnderBusinessObjects == null) {
			packageUnderBusinessObjects = getPackageAboveBusinessObjects() + "." + "businessobjects" + "."
					+ getLastPackageName();
		}
		return packageUnderBusinessObjects;
	}

	public String getPackageUnderDAOs() {
		if (packageUnderDAOs == null) {
			packageUnderDAOs = getPackageAboveBusinessObjects() + "." + "daos" + "." + getLastPackageName();
		}
		return packageUnderDAOs;
	}

	public String getPackageUnderEvents() {
		if (packageUnderEvents == null) {
			packageUnderEvents = getPackageAboveBusinessObjects() + "." + "businessobjects.events";
		}
		return packageUnderEvents;
	}

	public String getPackageUnderAdministrator() {
		if (packageUnderAdministrator == null) {
			packageUnderAdministrator = getPackageAboveBusinessObjects() + "." + "administrator" + "."
					+ getLastPackageName();
		}
		return packageUnderAdministrator;
	}

	public String getPathToPackageAboveBusinessObjects() {
		if (pathToPackageAboveBusinessObjects == null) {
			String[] packagesAndClassName = getFullyQualifiedName().split("\\.");
			pathToPackageAboveBusinessObjects = System.getProperty("file.separator") + "src"
					+ System.getProperty("file.separator");
			for (int i = 0; i < packagesAndClassName.length - 3; i++) {
				String packageName = packagesAndClassName[i];
				pathToPackageAboveBusinessObjects = pathToPackageAboveBusinessObjects + packageName
						+ System.getProperty("file.separator");
			}
		}
		return pathToPackageAboveBusinessObjects;
	}

	public String getPathToPackageUnderBusinessObjects() {
		if (pathToPackageUnderBusinessObjects == null) {
			pathToPackageUnderBusinessObjects = getPathToPackageAboveBusinessObjects() + "businessobjects"
					+ System.getProperty("file.separator") + getLastPackageName()
					+ System.getProperty("file.separator");
		}
		return pathToPackageUnderBusinessObjects;
	}

	public String getPathToPackageUnderEvents() {
		if (pathToPackageUnderEvents == null) {
			pathToPackageUnderEvents = getPathToPackageAboveBusinessObjects() + "businessobjects"
					+ System.getProperty("file.separator") + "events" + System.getProperty("file.separator");
		}
		return pathToPackageUnderEvents;
	}

	public String getPathToPackageUnderDAOs() {
		if (pathToPackageUnderDAOs == null) {
			pathToPackageUnderDAOs = this.getPathToPackageAboveBusinessObjects() + "daos"
					+ System.getProperty("file.separator") + getLastPackageName()
					+ System.getProperty("file.separator");
		}
		return pathToPackageUnderDAOs;
	}

	public String getPathToPackageUnderAdministrator() {
		if (pathToPackageUnderAdministrator == null) {
			pathToPackageUnderAdministrator = this.getPathToPackageAboveBusinessObjects() + "administrator"
					+ System.getProperty("file.separator") + getLastPackageName()
					+ System.getProperty("file.separator");
		}
		return pathToPackageUnderAdministrator;
	}

	public String getPathToDefaultAdministratorJSPs() {
		if (pathToDefaultAdministratorJSPs == null) {
			String fs = System.getProperty("file.separator");
			pathToDefaultAdministratorJSPs = fs + "templates" + fs + "administrator" + fs + "default" + fs
					+ getLastPackageName() + fs;
		}
		return pathToDefaultAdministratorJSPs;
	}

	public String getPathToCustomAdministratorJSPs() {
		if (pathToCustomAdministratorJSPs == null) {
			String fs = System.getProperty("file.separator");
			pathToCustomAdministratorJSPs = fs + "templates" + fs + "administrator" + fs + "custom" + fs
					+ getLastPackageName() + fs;
		}
		return pathToCustomAdministratorJSPs;
	}

	public String getPathToCustomCoreAdministratorJSPs() {
		if (pathToCustomCoreAdministratorJSPs == null) {
			String fs = System.getProperty("file.separator");
			pathToCustomCoreAdministratorJSPs = fs + "templates" + fs + "administrator" + fs + "custom" + fs + "core"
					+ fs;
		}
		return pathToCustomCoreAdministratorJSPs;
	}

	public String getPathToResourcesSubdirectory() {
		if (pathToResourcesSubdirectory == null) {
			pathToResourcesSubdirectory = System.getProperty("file.separator") + "src"
					+ System.getProperty("file.separator") + "resources" + System.getProperty("file.separator")
					+ getLastPackageName() + System.getProperty("file.separator");
		}
		return pathToResourcesSubdirectory;
	}

	public boolean doesClassExist() {
		try {
			Class.forName(getFullyQualifiedName());
		} catch (ClassNotFoundException cnfe) {
			return false;
		}
		return true;
	}

	public boolean isClassInComponents() {
		for (String key : getAppComponents().stringPropertyNames()) {
			String value = getAppComponents().getProperty(key);
			if (getFullyQualifiedName().equals(value)) {
				return true;
			}
		}
		return false;
	}

	public boolean isCustomEntity() {
		return !getFullyQualifiedName().startsWith("com.softslate.commerce");
	}

	public String getIncomingName() {
		return incomingName;
	}

	public void setIncomingName(String incomingName) {
		this.incomingName = incomingName;
	}

	public Properties getAppComponents() {
		return appComponents;
	}

	public void setAppComponents(Properties appComponents) {
		this.appComponents = appComponents;
	}

	public String getDbTableName() {
		if (dbTableName == null) {
			dbTableName = "ssc" + getInterfaceName();
		}
		return dbTableName;
	}

	public String getNameField() {
		if (nameField == null) {
			nameField = "name";
		}
		return nameField;
	}

	public void setNameField(String nameField) {
		this.nameField = nameField;
	}

	public String getCappedNameWithSpaces() {
		if (cappedNameWithSpaces == null) {
			cappedNameWithSpaces = addSpaces(getCappedName());
		}
		return cappedNameWithSpaces;
	}

	public String getCappedNameWithSpacesShortened() {
		if (cappedNameWithSpacesShortened == null) {
			cappedNameWithSpacesShortened = shorten(getCappedNameWithSpaces());
		}
		return cappedNameWithSpacesShortened;
	}

	public String getLowerNameWithSpaces() {
		if (lowerNameWithSpaces == null) {
			lowerNameWithSpaces = getCappedNameWithSpaces().toLowerCase();
		}
		return lowerNameWithSpaces;
	}

	public String getCappedPluralNameWithSpaces() {
		if (cappedPluralNameWithSpaces == null) {
			cappedPluralNameWithSpaces = pluralize(getCappedNameWithSpaces());
		}
		return cappedPluralNameWithSpaces;
	}

	public String getLowerPluralNameWithSpaces() {
		if (lowerPluralNameWithSpaces == null) {
			lowerPluralNameWithSpaces = pluralize(getLowerNameWithSpaces());
		}
		return lowerPluralNameWithSpaces;
	}

	public String getPathToCustomStrutsConfFile() {
		if (pathToCustomStrutsConfFile == null) {
			String fs = System.getProperty("file.separator");
			pathToCustomStrutsConfFile = fs + "conf" + fs + "administrator" + fs + "core" + fs
					+ "struts-config-custom.xml";
		}
		return pathToCustomStrutsConfFile;
	}

	public String getPathToCustomTilesConfFile() {
		if (pathToCustomTilesConfFile == null) {
			String fs = System.getProperty("file.separator");
			pathToCustomTilesConfFile = fs + "conf" + fs + "administrator" + fs + "core" + fs + "tiles-defs-custom.xml";
		}
		return pathToCustomTilesConfFile;
	}

	public static String pluralize(String s) {
		String ret = s;
		if (s.endsWith("[^aeiou]y")) {
			ret = s.substring(0, s.length() - 1) + "ies";
		} else {
			ret = s + "s";
		}
		return ret;
	}

	public static String addSpaces(String s) {
		StringBuffer sb = new StringBuffer("");
		for (int i = 0; i < s.length(); i++) {
			char c = s.charAt(i);
			if (i > 0 && Character.isUpperCase(c)) {
				sb.append(" ");
			}
			sb.append(c);
		}
		return sb.toString();
	}

	public static String shorten(String s) {
		if (s != null && s.length() > 18) {
			s = s.substring(0, 18) + "...";
		}
		return s;
	}

}
