package com.softslate.commerce.businessobjects.customer;

import java.io.Serializable;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BaseBusinessObject;

/**
 * Object representing a customer's delivery address within the system.
 * <code>CustomerAddressBean</code> is the default implementation of the
 * <code>CustomerAddress</code> interface for the application.
 * <p>
 * The database stores customer addresses in the <code>sscCustomerAddress</code>
 * table, and this interface can be used to represent a single row of that
 * table. The Hibernate configuration file
 * <code>CustomerAddressBean.hbm.xml</code> maps that table's columns to the
 * properties in this object.
 * 
 * @author David Tobey
 * 
 * @see com.softslate.commerce.businessobjects.customer.CustomerAddress
 */
@Audited
@AuditTable("AudCustomerAddress")
public class CustomerAddressBean extends BaseBusinessObject implements
		CustomerAddress, Serializable {

	private static final long serialVersionUID = 3257288011124125745L;

	static Log log = LogFactory.getLog(CustomerAddressBean.class);

	private Customer customer = null;

	private int customerAddressID = 0;

	public int getCustomerAddressID() {
		return (this.customerAddressID);
	}

	public void setCustomerAddressID(int customerAddressID) {
		this.customerAddressID = customerAddressID;
	}

	private int customerID;

	public int getCustomerID() {
		return (this.customerID);
	}

	public void setCustomerID(int customerID) {
		this.customerID = customerID;
	}

	private String status = null;

	public String getStatus() {
		return (this.status);
	}

	public void setStatus(String status) {
		this.status = status;
	}

	private String firstName = null;

	public String getFirstName() {
		return (this.firstName);
	}

	public void setFirstName(String firstName) {
		this.firstName = firstName;
	}

	private String lastName = null;

	public String getLastName() {
		return (this.lastName);
	}

	public void setLastName(String lastName) {
		this.lastName = lastName;
	}

	private String organization = null;

	public String getOrganization() {
		return (this.organization);
	}

	public void setOrganization(String organization) {
		this.organization = organization;
	}

	private String address1 = null;

	public String getAddress1() {
		return (this.address1);
	}

	public void setAddress1(String address1) {
		this.address1 = address1;
	}

	private String address2 = null;

	public String getAddress2() {
		return (this.address2);
	}

	public void setAddress2(String address2) {
		this.address2 = address2;
	}

	private String city = null;

	public String getCity() {
		return (this.city);
	}

	public void setCity(String city) {
		this.city = city;
	}

	private String state = null;

	public String getState() {
		return (this.state);
	}

	public void setState(String state) {
		this.state = state;
	}

	private String country = null;

	public String getCountry() {
		return (this.country);
	}

	public void setCountry(String country) {
		this.country = country;
	}

	private String postalCode = null;

	public String getPostalCode() {
		return (this.postalCode);
	}

	public void setPostalCode(String postalCode) {
		this.postalCode = postalCode;
	}

	private String phone1 = null;

	public String getPhone1() {
		return (this.phone1);
	}

	public void setPhone1(String phone1) {
		this.phone1 = phone1;
	}

	private String phone2 = null;

	public String getPhone2() {
		return (this.phone2);
	}

	public void setPhone2(String phone2) {
		this.phone2 = phone2;
	}

	private String email1 = null;

	public String getEmail1() {
		return (this.email1);
	}

	public void setEmail1(String email1) {
		this.email1 = email1;
	}

	private String email2 = null;

	public String getEmail2() {
		return (this.email2);
	}

	public void setEmail2(String email2) {
		this.email2 = email2;
	}

	private String extra1 = null;

	public String getExtra1() {
		return (this.extra1);
	}

	public void setExtra1(String extra1) {
		this.extra1 = extra1;
	}

	private String extra2 = null;

	private String extra3 = null;

	private String extra4 = null;

	private String extra5 = null;

	public String getExtra2() {
		return (this.extra2);
	}

	public void setExtra2(String extra2) {
		this.extra2 = extra2;
	}

	public String getExtra3() {
		return extra3;
	}

	public void setExtra3(String extra3) {
		this.extra3 = extra3;
	}

	public String getExtra4() {
		return extra4;
	}

	public void setExtra4(String extra4) {
		this.extra4 = extra4;
	}

	public String getExtra5() {
		return extra5;
	}

	public void setExtra5(String extra5) {
		this.extra5 = extra5;
	}

	private String notes = null;

	public String getNotes() {
		return (this.notes);
	}

	public void setNotes(String notes) {
		this.notes = notes;
	}

	// Aliases to the above getters and setters...

	public String getDeliveryFirstName() {
		return (getFirstName());
	}

	public void setDeliveryFirstName(String deliveryFirstName) {
		setFirstName(deliveryFirstName);
	}

	public String getDeliveryLastName() {
		return (getLastName());
	}

	public void setDeliveryLastName(String deliveryLastName) {
		setLastName(deliveryLastName);
	}

	public String getDeliveryOrganization() {
		return (getOrganization());
	}

	public void setDeliveryOrganization(String deliveryOrganization) {
		setOrganization(deliveryOrganization);
	}

	public String getDeliveryAddress1() {
		return (getAddress1());
	}

	public void setDeliveryAddress1(String deliveryAddress1) {
		setAddress1(deliveryAddress1);
	}

	public String getDeliveryAddress2() {
		return (getAddress2());
	}

	public void setDeliveryAddress2(String deliveryAddress2) {
		setAddress2(deliveryAddress2);
	}

	public String getDeliveryCity() {
		return (getCity());
	}

	public void setDeliveryCity(String deliveryCity) {
		setCity(deliveryCity);
	}

	public String getDeliveryState() {
		return (getState());
	}

	public void setDeliveryState(String deliveryState) {
		setState(deliveryState);
	}

	public String getDeliveryCountry() {
		return (getCountry());
	}

	public void setDeliveryCountry(String deliveryCountry) {
		setCountry(deliveryCountry);
	}

	public String getDeliveryPostalCode() {
		return (getPostalCode());
	}

	public void setDeliveryPostalCode(String deliveryPostalCode) {
		setPostalCode(deliveryPostalCode);
	}

	public String getDeliveryPhone1() {
		return (getPhone1());
	}

	public void setDeliveryPhone1(String deliveryPhone1) {
		setPhone1(deliveryPhone1);
	}

	public String getDeliveryPhone2() {
		return (getPhone2());
	}

	public void setDeliveryPhone2(String deliveryPhone2) {
		setPhone2(deliveryPhone2);
	}

	public String getDeliveryEmail1() {
		return (getEmail1());
	}

	public void setDeliveryEmail1(String deliveryEmail1) {
		setEmail1(deliveryEmail1);
	}

	public String getDeliveryEmail2() {
		return (getEmail2());
	}

	public void setDeliveryEmail2(String deliveryEmail2) {
		setEmail2(deliveryEmail2);
	}

	public String getDeliveryExtra1() {
		return (getExtra1());
	}

	public void setDeliveryExtra1(String deliveryExtra1) {
		setExtra1(deliveryExtra1);
	}

	public String getDeliveryExtra2() {
		return (getExtra2());
	}

	public void setDeliveryExtra2(String deliveryExtra2) {
		setExtra2(deliveryExtra2);
	}

	public String getDeliveryExtra3() {
		return (getExtra3());
	}

	public void setDeliveryExtra3(String deliveryExtra3) {
		setExtra3(deliveryExtra3);
	}

	public String getDeliveryExtra4() {
		return (getExtra4());
	}

	public void setDeliveryExtra4(String deliveryExtra4) {
		setExtra4(deliveryExtra4);
	}

	public String getDeliveryExtra5() {
		return (getExtra5());
	}

	public void setDeliveryExtra5(String deliveryExtra5) {
		setExtra5(deliveryExtra5);
	}

	public String getDeliveryNotes() {
		return (getNotes());
	}

	public void setDeliveryNotes(String deliveryNotes) {
		setNotes(deliveryNotes);
	}

	public Customer getCustomer() {
		return customer;
	}

	public void setCustomer(Customer customer) {
		this.customer = customer;
	}

	private boolean isDefault = true;

	public boolean getIsDefault() {
		return (this.isDefault);
	}

	public void setIsDefault(boolean isDefault) {
		this.isDefault = isDefault;
	}

	private String paymentToken = null;

	public String getPaymentToken() {
		return paymentToken;
	}

	public void setPaymentToken(String paymentToken) {
		this.paymentToken = paymentToken;
	}

	private String creditCardName = null;

	public String getCreditCardName() {
		return creditCardName;
	}

	public void setCreditCardName(String creditCardName) {
		this.creditCardName = creditCardName;
	}
	private String creditCardType = null;

	public String getCreditCardType() {
		return creditCardType;
	}

	public void setCreditCardType(String creditCardType) {
		this.creditCardType = creditCardType;
	}
	private String creditCardDate = null;

	public String getCreditCardDate() {
		return creditCardDate;
	}

	public void setCreditCardDate(String creditCardDate) {
		this.creditCardDate = creditCardDate;
	}
	private String creditCardNumberDisplay = null;

	public String getCreditCardNumberDisplay() {
		return creditCardNumberDisplay;
	}

	public void setCreditCardNumberDisplay(String creditCardNumberDisplay) {
		this.creditCardNumberDisplay = creditCardNumberDisplay;
	}

	private boolean isBilling = false;

	public boolean getIsBilling() {
		return isBilling;
	}

	public void setIsBilling(boolean isBilling) {
		this.isBilling = isBilling;
	}

	public String getBillingFirstName() {
		return (getFirstName());
	}

	public void setBillingFirstName(String billingFirstName) {
		setFirstName(billingFirstName);
	}

	public String getBillingLastName() {
		return (getLastName());
	}

	public void setBillingLastName(String billingLastName) {
		setLastName(billingLastName);
	}

	public String getBillingOrganization() {
		return (getOrganization());
	}

	public void setBillingOrganization(String billingOrganization) {
		setOrganization(billingOrganization);
	}

	public String getBillingAddress1() {
		return (getAddress1());
	}

	public void setBillingAddress1(String billingAddress1) {
		setAddress1(billingAddress1);
	}

	public String getBillingAddress2() {
		return (getAddress2());
	}

	public void setBillingAddress2(String billingAddress2) {
		setAddress2(billingAddress2);
	}

	public String getBillingCity() {
		return (getCity());
	}

	public void setBillingCity(String billingCity) {
		setCity(billingCity);
	}

	public String getBillingState() {
		return (getState());
	}

	public void setBillingState(String billingState) {
		setState(billingState);
	}

	public String getBillingCountry() {
		return (getCountry());
	}

	public void setBillingCountry(String billingCountry) {
		setCountry(billingCountry);
	}

	public String getBillingPostalCode() {
		return (getPostalCode());
	}

	public void setBillingPostalCode(String billingPostalCode) {
		setPostalCode(billingPostalCode);
	}

	public String getBillingPhone1() {
		return (getPhone1());
	}

	public void setBillingPhone1(String billingPhone1) {
		setPhone1(billingPhone1);
	}

	public String getBillingPhone2() {
		return (getPhone2());
	}

	public void setBillingPhone2(String billingPhone2) {
		setPhone2(billingPhone2);
	}

	public String getBillingEmail1() {
		return (getEmail1());
	}

	public void setBillingEmail1(String billingEmail1) {
		setEmail1(billingEmail1);
	}

	public String getBillingEmail2() {
		return (getEmail2());
	}

	public void setBillingEmail2(String billingEmail2) {
		setEmail2(billingEmail2);
	}

	public String getBillingExtra1() {
		return (getExtra1());
	}

	public void setBillingExtra1(String billingExtra1) {
		setExtra1(billingExtra1);
	}

	public String getBillingExtra2() {
		return (getExtra2());
	}

	public void setBillingExtra2(String billingExtra2) {
		setExtra2(billingExtra2);
	}

	public String getBillingExtra3() {
		return (getExtra3());
	}

	public void setBillingExtra3(String billingExtra3) {
		setExtra3(billingExtra3);
	}

	public String getBillingExtra4() {
		return (getExtra4());
	}

	public void setBillingExtra4(String billingExtra4) {
		setExtra4(billingExtra4);
	}

	public String getBillingExtra5() {
		return (getExtra5());
	}

	public void setBillingExtra5(String billingExtra5) {
		setExtra5(billingExtra5);
	}

	public String getBillingNotes() {
		return (getNotes());
	}

	public void setBillingNotes(String billingNotes) {
		setNotes(billingNotes);
	}

	private String identifier = null;
	
	public String getIdentifier() {
		return identifier;
	}

	public void setIdentifier(String identifier) {
		this.identifier = identifier;
	}

}
