package com.softslate.commerce.businessobjects.order;

import java.util.Map;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.core.BaseBusinessProcessor;
import com.softslate.commerce.daos.order.OrderItemAttributeDAO;
import com.softslate.commerce.daos.order.OrderItemAttributeGatewayDAO;

/**
 * Processes business logic having to do with order item attributes. Created in
 * the Struts layer in response to requests for viewing, editing, adding, and
 * deleting order item attributes.
 * <p>
 * <code>BasicOrderItemAttributeProcessor</code> is the default implementation
 * of the <code>OrderItemAttributeProcessor</code> interface for the
 * application.
 * 
 * @author David Tobey
 * 
 */
public class BasicOrderItemAttributeProcessor extends BaseBusinessProcessor
        implements OrderItemAttributeProcessor {

    static Log log = LogFactory.getLog(BasicOrderItemAttributeProcessor.class);

    public Map loadOrderItemAttributesAndCount(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null)
            log.info(getUser().getAdministrator().getUserName()
                    + ": Loading order item attributes and count");
        OrderItemAttributeGatewayDAO gatewayDAO =  getDaoFactory()
                .createDAO(OrderItemAttributeGatewayDAO.class);
        return gatewayDAO.loadOrderItemAttributesAndCount(parameters);
    }

    public Map addOrderItemAttribute(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
            String code = (String) parameters.get("attributeCode");
            log.info(getUser().getAdministrator().getUserName()
                    + ": Adding order item attribute: " + code);
        }
        OrderItemAttribute orderItemAttribute =  getBusinessObjectFactory()
                .createObject(OrderItemAttribute.class);
        BeanUtils.copyProperties(orderItemAttribute, parameters);
        OrderItem orderItem =  getBusinessObjectFactory()
                .createObject(OrderItem.class);
        orderItem.setOrderItemID(orderItemAttribute.getOrderItemID());
        orderItemAttribute.setOrderItem(orderItem);
        OrderItemAttributeDAO dao =  getDaoFactory()
                .createDAO(OrderItemAttributeDAO.class);
        dao.setOrderItemAttribute(orderItemAttribute);
        dao.insertOrderItemAttribute(true);
        return null;
    }

    public OrderItemAttribute loadOrderItemAttributeFromID(Map parameters)
            throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null)
            log.info(getUser().getAdministrator().getUserName()
                    + ": Loading order item attribute from id");
        OrderItemAttribute orderItemAttribute =  getBusinessObjectFactory()
                .createObject(OrderItemAttribute.class);
        OrderItemAttributeDAO dao =  getDaoFactory()
                .createDAO(OrderItemAttributeDAO.class);
        BeanUtils.copyProperties(orderItemAttribute, parameters);
        dao.setOrderItemAttribute(orderItemAttribute);
        dao.loadOrderItemAttributeFromID();
        return orderItemAttribute;
    }

    public Map editOrderItemAttribute(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
            String code = (String) parameters.get("attributeCode");
            log.info(getUser().getAdministrator().getUserName()
                    + ": Editing order item attribute: " + code);
        }
        OrderItemAttribute orderItemAttribute =  getBusinessObjectFactory()
                .createObject(OrderItemAttribute.class);
        BeanUtils.copyProperties(orderItemAttribute, parameters);
        OrderItem orderItem =  getBusinessObjectFactory()
                .createObject(OrderItem.class);
        orderItem.setOrderItemID(orderItemAttribute.getOrderItemID());
        orderItemAttribute.setOrderItem(orderItem);
        OrderItemAttributeDAO dao =  getDaoFactory()
                .createDAO(OrderItemAttributeDAO.class);
        dao.setOrderItemAttribute(orderItemAttribute);
        dao.updateOrderItemAttribute(true);
        return null;
    }

	public Map editDeleteOrderItemAttributes(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Editing/deleting multiple order item attributes");
		OrderItemAttributeGatewayDAO dao = (OrderItemAttributeGatewayDAO) getDaoFactory().createDAO(
				"orderItemAttributeGatewayDAOImplementer");
		Map results = dao.editDeleteOrderItemAttributes(parameters, getBusinessObjectFactory());
		return results;
	}

    public void deleteOrderItemAttribute(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
            String code = (String) parameters.get("attributeCode");
            log.info(getUser().getAdministrator().getUserName()
                    + ": Delete order item attribute: " + code);
        }
        OrderItemAttribute orderItemAttribute =  getBusinessObjectFactory()
                .createObject(OrderItemAttribute.class);
        BeanUtils.copyProperties(orderItemAttribute, parameters);
        OrderItemAttributeDAO dao =  getDaoFactory()
                .createDAO(OrderItemAttributeDAO.class);
        dao.setOrderItemAttribute(orderItemAttribute);
        dao.deleteOrderItemAttribute(true);
        return;
    }
}
