package com.softslate.commerce.businessobjects.order;

import java.io.Serializable;
import java.text.NumberFormat;
import java.util.Collection;
import java.util.Currency;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;

import com.softslate.commerce.businessobjects.core.BaseBusinessObject;
import com.softslate.commerce.businessobjects.customer.Customer;

/**
 * Object representing an order within the system. <code>OrderBean</code> is
 * the default implementation of the <code>Order</code> interface for the
 * application.
 * <p>
 * The database stores orders in the <code>sscOrder</code> table, and this
 * interface can be used to represent a single row of that table. The Hibernate
 * configuration file <code>OrderBean.hbm.xml</code> maps that table's columns
 * to the properties in this object.
 * 
 * @author David Tobey
 * 
 * @see com.softslate.commerce.businessobjects.order.Order
 */
@Audited
@AuditTable("AudOrder")
public class OrderBean extends BaseBusinessObject implements Order,
		Serializable {

	private static final long serialVersionUID = 4120852148835070003L;

	static Log log = LogFactory.getLog(OrderBean.class);

	private int orderID = 0;

	public int getOrderID() {
		return (this.orderID);
	}

	public void setOrderID(int orderID) {
		this.orderID = orderID;
	}

	private Integer customerID = null;

	public Integer getCustomerID() {
		return (this.customerID);
	}

	public void setCustomerID(Integer customerID) {
		this.customerID = customerID;
	}

	private String userName = null;

	public String getUserName() {
		return (this.userName);
	}

	public void setUserName(String userName) {
		this.userName = userName;
	}

	private Integer orderNumber = null;

	public Integer getOrderNumber() {
		return (this.orderNumber);
	}

	public void setOrderNumber(Integer orderNumber) {
		this.orderNumber = orderNumber;
	}

	private String trackingNumber;

	public String getTrackingNumber() {
		return (this.trackingNumber);
	}

	public void setTrackingNumber(String trackingNumber) {
		this.trackingNumber = trackingNumber;
	}

	private Double total;

	public Double getTotal() {
		return (this.total);
	}

	public void setTotal(Double total) {
		this.total = total;
	}

	private Double shipping;

	public Double getShipping() {
		return (this.shipping);
	}

	public void setShipping(Double shipping) {
		this.shipping = shipping;
	}

	private Double tax;

	public Double getTax() {
		return (this.tax);
	}

	public void setTax(Double tax) {
		this.tax = tax;
	}

	private Double taxableSubtotal;

	public Double getTaxableSubtotal() {
		return (this.taxableSubtotal);
	}

	public void setTaxableSubtotal(Double taxableSubtotal) {
		this.taxableSubtotal = taxableSubtotal;
	}

	private Double subtotal;

	public Double getSubtotal() {
		return (this.subtotal);
	}

	public void setSubtotal(Double subtotal) {
		this.subtotal = subtotal;
	}

	private Double weight;

	public Double getWeight() {
		return (this.weight);
	}

	public void setWeight(Double weight) {
		this.weight = weight;
	}

	private Integer quantity;

	public Integer getQuantity() {
		return (this.quantity);
	}

	public void setQuantity(Integer quantity) {
		this.quantity = quantity;
	}

	private String formattedTotal;

	public String getFormattedTotal() {
		return (this.formattedTotal);
	}

	public void setFormattedTotal(String formattedTotal) {
		this.formattedTotal = formattedTotal;
	}

	private String formattedShipping;

	public String getFormattedShipping() {
		return (this.formattedShipping);
	}

	public void setFormattedShipping(String formattedShipping) {
		this.formattedShipping = formattedShipping;
	}

	private String formattedTax;

	public String getFormattedTax() {
		return (this.formattedTax);
	}

	public void setFormattedTax(String formattedTax) {
		this.formattedTax = formattedTax;
	}

	private String formattedTaxableSubtotal;

	public String getFormattedTaxableSubtotal() {
		return (this.formattedTaxableSubtotal);
	}

	public void setFormattedTaxableSubtotal(String formattedTaxableSubtotal) {
		this.formattedTaxableSubtotal = formattedTaxableSubtotal;
	}

	private String formattedSubtotal;

	public String getFormattedSubtotal() {
		return (this.formattedSubtotal);
	}

	public void setFormattedSubtotal(String formattedSubtotal) {
		this.formattedSubtotal = formattedSubtotal;
	}

	private String created = null;

	public String getCreated() {
		return (this.created);
	}

	public void setCreated(String created) {
		this.created = created;
	}

	private String lastModified = null;

	public String getLastModified() {
		return (this.lastModified);
	}

	public void setLastModified(String lastModified) {
		this.lastModified = lastModified;
	}

	private String completed = null;

	public String getCompleted() {
		return (this.completed);
	}

	public void setCompleted(String completed) {
		this.completed = completed;
	}

	private String formattedCreated = null;

	public String getFormattedCreated() {
		return (this.formattedCreated);
	}

	public void setFormattedCreated(String formattedCreated) {
		this.formattedCreated = formattedCreated;
	}

	private String formattedLastModified = null;

	public String getFormattedLastModified() {
		return (this.formattedLastModified);
	}

	public void setFormattedLastModified(String formattedLastModified) {
		this.formattedLastModified = formattedLastModified;
	}

	private String formattedCompleted = null;

	public String getFormattedCompleted() {
		return (this.formattedCompleted);
	}

	public void setFormattedCompleted(String formattedCompleted) {
		this.formattedCompleted = formattedCompleted;
	}

	private String status = null;

	public String getStatus() {
		return (this.status);
	}

	public void setStatus(String status) {
		this.status = status;
	}

	private String statusDetails = null;

	public String getStatusDetails() {
		return (this.statusDetails);
	}

	public void setStatusDetails(String statusDetails) {
		this.statusDetails = statusDetails;
	}

	private String shippingMethod = null;

	public String getShippingMethod() {
		return shippingMethod;
	}

	public void setShippingMethod(String shippingMethod) {
		this.shippingMethod = shippingMethod;
	}

	private String firstName = null;

	public String getFirstName() {
		return (this.firstName);
	}

	public void setFirstName(String firstName) {
		this.firstName = firstName;
	}

	private String lastName = null;

	public String getLastName() {
		return (this.lastName);
	}

	public void setLastName(String lastName) {
		this.lastName = lastName;
	}

	private String organization = null;

	public String getOrganization() {
		return (this.organization);
	}

	public void setOrganization(String organization) {
		this.organization = organization;
	}

	private String address1 = null;

	public String getAddress1() {
		return (this.address1);
	}

	public void setAddress1(String address1) {
		this.address1 = address1;
	}

	private String address2 = null;

	public String getAddress2() {
		return (this.address2);
	}

	public void setAddress2(String address2) {
		this.address2 = address2;
	}

	private String city = null;

	public String getCity() {
		return (this.city);
	}

	public void setCity(String city) {
		this.city = city;
	}

	private String state = null;

	public String getState() {
		return (this.state);
	}

	public void setState(String state) {
		this.state = state;
	}

	private String country = null;

	public String getCountry() {
		return (this.country);
	}

	public void setCountry(String country) {
		this.country = country;
	}

	private String postalCode = null;

	public String getPostalCode() {
		return (this.postalCode);
	}

	public void setPostalCode(String postalCode) {
		this.postalCode = postalCode;
	}

	private String phone1 = null;

	public String getPhone1() {
		return (this.phone1);
	}

	public void setPhone1(String phone1) {
		this.phone1 = phone1;
	}

	private String phone2 = null;

	public String getPhone2() {
		return (this.phone2);
	}

	public void setPhone2(String phone2) {
		this.phone2 = phone2;
	}

	private String email1 = null;

	public String getEmail1() {
		return (this.email1);
	}

	public void setEmail1(String email1) {
		this.email1 = email1;
	}

	private String email2 = null;

	public String getEmail2() {
		return (this.email2);
	}

	public void setEmail2(String email2) {
		this.email2 = email2;
	}

	private String extra1 = null;

	public String getExtra1() {
		return (this.extra1);
	}

	public void setExtra1(String extra1) {
		this.extra1 = extra1;
	}

	private String extra2 = null;

	public String getExtra2() {
		return (this.extra2);
	}

	public void setExtra2(String extra2) {
		this.extra2 = extra2;
	}

	private String extra3 = null;

	public String getExtra3() {
		return (this.extra3);
	}

	public void setExtra3(String extra3) {
		this.extra3 = extra3;
	}

	private String extra4 = null;

	public String getExtra4() {
		return (this.extra4);
	}

	public void setExtra4(String extra4) {
		this.extra4 = extra4;
	}

	private String extra5 = null;

	public String getExtra5() {
		return (this.extra5);
	}

	public void setExtra5(String extra5) {
		this.extra5 = extra5;
	}

	private String notes = null;

	public String getNotes() {
		return (this.notes);
	}

	public void setNotes(String notes) {
		this.notes = notes;
	}

	private String autoSavedCartToken = null;

	public String getAutoSavedCartToken() {
		return autoSavedCartToken;
	}

	public void setAutoSavedCartToken(String autoSavedCartToken) {
		this.autoSavedCartToken = autoSavedCartToken;
	}

	private Collection orderDeliveryCollection = null;

	public Collection getOrderDeliveryCollection() {
		return (this.orderDeliveryCollection);
	}

	public void setOrderDeliveryCollection(Collection orderDeliveryCollection) {
		this.orderDeliveryCollection = orderDeliveryCollection;
	}

	@NotAudited
	private Collection orderItemCollection = null;

	public Collection getOrderItemCollection() {
		return (this.orderItemCollection);
	}

	public void setOrderItemCollection(Collection orderItemCollection) {
		this.orderItemCollection = orderItemCollection;
	}

	private Collection payments = null;

	public Collection getPayments() {
		return payments;
	}

	public void setPayments(Collection payments) {
		this.payments = payments;
	}

	private Collection orderDiscounts = null;

	public Collection getOrderDiscounts() {
		return orderDiscounts;
	}

	public void setOrderDiscounts(Collection orderDiscounts) {
		this.orderDiscounts = orderDiscounts;
	}

	private Customer customer = null;

	public Customer getCustomer() {
		return customer;
	}

	public void setCustomer(Customer customer) {
		this.customer = customer;
	}

	// Aliases for the above getters and setters...

	public String getBillingFirstName() {
		return (getFirstName());
	}

	public void setBillingFirstName(String billingFirstName) {
		setFirstName(billingFirstName);
	}

	public String getBillingLastName() {
		return (getLastName());
	}

	public void setBillingLastName(String billingLastName) {
		setLastName(billingLastName);
	}

	public String getBillingOrganization() {
		return (getOrganization());
	}

	public void setBillingOrganization(String billingOrganization) {
		setOrganization(billingOrganization);
	}

	public String getBillingAddress1() {
		return (getAddress1());
	}

	public void setBillingAddress1(String billingAddress1) {
		setAddress1(billingAddress1);
	}

	public String getBillingAddress2() {
		return (getAddress2());
	}

	public void setBillingAddress2(String billingAddress2) {
		setAddress2(billingAddress2);
	}

	public String getBillingCity() {
		return (getCity());
	}

	public void setBillingCity(String billingCity) {
		setCity(billingCity);
	}

	public String getBillingState() {
		return (getState());
	}

	public void setBillingState(String billingState) {
		setState(billingState);
	}

	public String getBillingCountry() {
		return (getCountry());
	}

	public void setBillingCountry(String billingCountry) {
		setCountry(billingCountry);
	}

	public String getBillingPostalCode() {
		return (getPostalCode());
	}

	public void setBillingPostalCode(String billingPostalCode) {
		setPostalCode(billingPostalCode);
	}

	public String getBillingPhone1() {
		return (getPhone1());
	}

	public void setBillingPhone1(String billingPhone1) {
		setPhone1(billingPhone1);
	}

	public String getBillingPhone2() {
		return (getPhone2());
	}

	public void setBillingPhone2(String billingPhone2) {
		setPhone2(billingPhone2);
	}

	public String getBillingEmail1() {
		return (getEmail1());
	}

	public void setBillingEmail1(String billingEmail1) {
		setEmail1(billingEmail1);
	}

	public String getBillingEmail2() {
		return (getEmail2());
	}

	public void setBillingEmail2(String billingEmail2) {
		setEmail2(billingEmail2);
	}

	public String getBillingExtra1() {
		return (getExtra1());
	}

	public void setBillingExtra1(String billingExtra1) {
		setExtra1(billingExtra1);
	}

	public String getBillingExtra2() {
		return (getExtra2());
	}

	public void setBillingExtra2(String billingExtra2) {
		setExtra2(billingExtra2);
	}

	public String getBillingExtra3() {
		return (getExtra3());
	}

	public void setBillingExtra3(String billingExtra3) {
		setExtra3(billingExtra3);
	}

	public String getBillingExtra4() {
		return (getExtra4());
	}

	public void setBillingExtra4(String billingExtra4) {
		setExtra4(billingExtra4);
	}

	public String getBillingExtra5() {
		return (getExtra5());
	}

	public void setBillingExtra5(String billingExtra5) {
		setExtra5(billingExtra5);
	}

	public String getBillingNotes() {
		return (getNotes());
	}

	public void setBillingNotes(String billingNotes) {
		setNotes(billingNotes);
	}

	public OrderDelivery firstDelivery() {
		Iterator iterator1 = getOrderDeliveryCollection().iterator();
		if (iterator1.hasNext()) {
			return (OrderDelivery) iterator1.next();
		} else {
			return null;
		}
	}

	public void addFormatting(Locale locale, String currencyCode) {
		if (currencyCode == null) {
			currencyCode = "USD";
		}
		NumberFormat nf = NumberFormat.getCurrencyInstance(locale);
		Currency currency = Currency.getInstance(currencyCode);
		nf.setCurrency(currency);
		if (getTotal() != null) {
			setFormattedTotal(nf.format(getTotal().doubleValue()));
		}
		if (getSubtotal() != null) {
			setFormattedSubtotal(nf.format(getSubtotal().doubleValue()));
		}
		if (getTax() != null) {
			setFormattedTax(nf.format(getTax().doubleValue()));
		}
		if (getTaxableSubtotal() != null) {
			setFormattedTaxableSubtotal(nf.format(getTaxableSubtotal()
					.doubleValue()));
		}
		if (getShipping() != null) {
			setFormattedShipping(nf.format(getShipping().doubleValue()));
		}
	}

	private Collection orderShippingRules = null;

	public Collection getOrderShippingRules() {
		return orderShippingRules;
	}

	public void setOrderShippingRules(Collection orderShippingRules) {
		this.orderShippingRules = orderShippingRules;
	}

	private Collection estimatedShippingOptions = null;

	public Collection getEstimatedShippingOptions() {
		return estimatedShippingOptions;
	}

	public void setEstimatedShippingOptions(Collection estimatedShippingOptions) {
		this.estimatedShippingOptions = estimatedShippingOptions;
	}

	private String paymentProfileID = null;

	public String getPaymentProfileID() {
		return paymentProfileID;
	}

	public void setPaymentProfileID(String paymentProfileID) {
		this.paymentProfileID = paymentProfileID;
	}

	private boolean skipAddressValidation = false;

	public boolean getSkipAddressValidation() {
		return skipAddressValidation;
	}

	public void setSkipAddressValidation(boolean skipAddressValidation) {
		this.skipAddressValidation = skipAddressValidation;
	}

	private Map orderSettings = null;

	public Map getOrderSettings() {
		return orderSettings;
	}

	public void setOrderSettings(Map orderSettings) {
		this.orderSettings = orderSettings;
	}

}
