package com.softslate.commerce.businessobjects.payment;

import java.util.HashMap;
import java.util.Map;

import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.businessobjects.payment.paypal.PayPalConstants;

public class PayPalUKDirectProcessor extends BasePayPalNVPProcessor {

	public Map process(Map parameters) throws Exception {
		if (log.isInfoEnabled())
			log.info("Processing payment.");

		Map result = new HashMap();

		String transactionType = (String) getSettings().getValue(
				"paypalUKPaymentType");

		String creditCardType = (String) parameters.get("creditCardType");
		if (null != creditCardType
				&& (creditCardType.equalsIgnoreCase("SWITCH") || creditCardType
						.equalsIgnoreCase("SOLO"))) {
			/*
			 * Note: From Paypal Integration Documentation
			 * ============================================ The Switch and Solo
			 * ACCTTYPEs do not support Delayed Capture transactions Hence,
			 * TRXTYPE = S (SALE) if card = switch or solo.
			 */
			transactionType = PayPalConstants.SALE_TRXTYPE;
		}

		String trxtype = transactionType;
		boolean doAuthentication = false;
		if (transactionType.equals(PayPalConstants.AUTH_AUTHENTICATE_TRXTYPE)
				|| transactionType
						.equals(PayPalConstants.AUTH_AUTHENTICATE_SALE_TRXTYPE)) {
			trxtype = PayPalConstants.AUTH_TRXTYPE;
			doAuthentication = true;
		}

		parameters.put(PayPalConstants.TRXTYPE, trxtype);
		parameters.put(PayPalConstants.TENDER,
				PayPalConstants.DIRECT_PAYMENT_TENDER);
		String response = this.makePayPalCall(parameters, "paypalUK");

		if (log.isDebugEnabled())
			log.debug("response: " + response);

		// Parse the response into a Map
		Map responseMap = parseResponseData(response);

		// Handle the response - errors first
		String resultCode = (String) responseMap.get(PayPalConstants.RESULT);
		if (resultCode == null) {
			result.put("resultCode", "10000");
			return result;
		} else if (!resultCode.equals("0")) {
			result.put("resultCode", resultCode);
			result.put("resultMessage", (String) responseMap
					.get(PayPalConstants.RESPMSG));
			return result;
		}

		// Successful response - if necessary, authenticate
		boolean authenticated = true;
		if (doAuthentication) {
			authenticated = isAuthenticated(responseMap);
		}

		// Send back error if can't authenticate
		if (!authenticated) {
//			String avsAddr = (String) responseMap
//					.get(PayPalConstants.AVS_ADDR_RESP);
//			String avsZip = (String) responseMap
//					.get(PayPalConstants.AVS_ZIP_RESP);
//			String cvv2 = (String) responseMap
//					.get(PayPalConstants.AVS_CVV2_RESP);
//			if (cvv2 == null)
//				cvv2 = "X";
//
//			result.put("resultCode", "10001");
//			if ((!avsAddr.equals("Y") || !avsZip.equals("Y"))
//					&& !cvv2.equals("Y")) {
//				result.put("resultCode", "10001");
//			} else if ((!avsAddr.equals("Y") || !avsZip.equals("Y"))
//					&& cvv2.equals("Y")) {
//				result.put("resultCode", "10002");
//			} else if ((avsAddr.equals("Y") && avsZip.equals("Y"))
//					&& !cvv2.equals("Y")) {
//				result.put("resultCode", "10003");
//			}

			// Only possibility is a failed security code.
			result.put("resultCode", "10003");

			if (log.isInfoEnabled())
				log.info("Failed authentication: " + result);

			return result;
		}

		// If transaction type is AAS (Authorize/Authenticate/Sale), run an
		// immediate delayed capture
		if (transactionType
				.equals(PayPalConstants.AUTH_AUTHENTICATE_SALE_TRXTYPE)) {
			// Update param map to use previous PNREF in a delayed capture
			parameters.put(PayPalConstants.PNREF, (String) responseMap
					.get(PayPalConstants.PNREF));
			parameters.put(PayPalConstants.TRXTYPE,
					PayPalConstants.CAPTURE_TRXTYPE);

			response = this.makePayPalCall(parameters, "paypalUK");

			if (log.isDebugEnabled())
				log.debug("response: " + response);
			// Parse the response into a Map
			responseMap = parseResponseData(response);

			// Handle any response errors
			resultCode = (String) responseMap.get(PayPalConstants.RESULT);
			if (resultCode == null) {
				result.put("resultCode", "10000");
				return result;
			} else if (!resultCode.equals("0")) {
				result.put("resultCode", resultCode);
				result.put("resultMessage", (String) responseMap
						.get(PayPalConstants.RESPMSG));
				return result;
			}
		}

		// Successful transaction - write it to the database
		responseMap.put("transactionType", transactionType);
		processSuccessfulTransaction(parameters, responseMap);

		result.put("resultCode", "0");
		return result;
	}

	public boolean isAuthenticated(Map responseMap) throws Exception {
		String cvv2 = (String) responseMap.get(PayPalConstants.AVS_CVV2_RESP);
		if (cvv2 == null)
			cvv2 = "X";
		String expectedCVV2 = (String) getSettings().getValue("paypalUKCVV2");
		if (expectedCVV2 != null && expectedCVV2.length() > 0
				&& expectedCVV2.indexOf(cvv2) == -1)
			return false;

//		String avsAddr = (String) responseMap
//				.get(PayPalConstants.AVS_ADDR_RESP);
//		String expectedAVSSt = (String) getSettings().getValue("paypalUKAVSSt");
//		if (expectedAVSSt != null && expectedAVSSt.length() > 0
//				&& expectedAVSSt.indexOf(avsAddr) == -1)
//			return false;
//
//		String avsZip = (String) responseMap.get(PayPalConstants.AVS_ZIP_RESP);
//		String expectedAVSZip = (String) getSettings().getValue(
//				"paypalUKAVSZip");
//		if (expectedAVSZip != null && expectedAVSZip.length() > 0
//				&& expectedAVSZip.indexOf(avsZip) == -1)
//			return false;

		return true;
	}

	protected Map preparePaypalRequestAcutalPayload(Map params, Map parameters) {
		// Prepare the request to PayPal Payflow Pro
		Order order = getUser().getOrder();

		if (null == params)
			params = new HashMap();

		String firstname = "";
		String lastname = "";
		String address = "";
		String city = "";
		String companyname = "";
		String state = "";
		String zip = "";
		String country = "";
		String email = "";
		String shiptofirstname = "";
		String shiptolastname = "";
		String nameToShip = "";
		String addressToShip = "";
		String cityToShip = "";
		String stateToShip = "";
		String zipToShip = "";
		if (order.getFirstName() != null)
			firstname = order.getFirstName();// todo get from parameters
		if (order.getLastName() != null)
			lastname = order.getLastName();// todo get from parameters
		if (order.getAddress1() != null)
			address = order.getAddress1();
		if (order.getAddress2() != null)
			address = address + " " + order.getAddress2();
		if (order.getCity() != null)
			city = order.getCity();
		if (order.getOrganization() != null)
			companyname = order.getOrganization();
		if (order.getState() != null)
			state = order.getState();
		if (order.getPostalCode() != null)
			zip = order.getPostalCode();
		if (order.getCountry() != null)
			country = order.getCountry();
		if (order.getEmail1() != null)
			email = order.getEmail1();
		if (order.firstDelivery().getFirstName() != null)
			shiptofirstname = order.firstDelivery().getFirstName();
		if (order.firstDelivery().getLastName() != null)
			shiptolastname = order.firstDelivery().getLastName();
		if (order.firstDelivery().getFirstName() != null)
			nameToShip = order.firstDelivery().getFirstName();
		if (order.firstDelivery().getLastName() != null)
			nameToShip = nameToShip + " " + order.firstDelivery().getLastName();
		if (order.firstDelivery().getAddress1() != null)
			addressToShip = order.firstDelivery().getAddress1();
		if (order.firstDelivery().getAddress2() != null)
			addressToShip = addressToShip + " "
					+ order.firstDelivery().getAddress2();
		if (order.firstDelivery().getCity() != null)
			cityToShip = order.firstDelivery().getCity();
		if (order.firstDelivery().getState() != null)
			stateToShip = order.firstDelivery().getState();
		if (order.firstDelivery().getPostalCode() != null)
			zipToShip = order.firstDelivery().getPostalCode();

		params.put(PayPalConstants.AMT, order.getTotal().toString());
		params.put(PayPalConstants.ITEM_AMT, order.getSubtotal().toString());
		params.put(PayPalConstants.TAXAMT, order.getTax().toString());
		params.put(PayPalConstants.FREIGHTAMT, order.getShipping().toString());
		params.put(PayPalConstants.ACCT, (String) parameters
				.get("decryptedNumber"));
		params.put(PayPalConstants.CVV2, (String) parameters
				.get("creditCardSecurityCode"));

		String creditCardExpirationYear = (String) parameters
				.get("creditCardExpirationYear");
		params.put(PayPalConstants.EXPDATE, (String) parameters
				.get("creditCardExpirationMonth")
				+ creditCardExpirationYear.substring(2, 4));

		String creditCardType = (String) parameters.get("creditCardType");
		if (null != creditCardType
				&& (creditCardType.equalsIgnoreCase("SWITCH") || creditCardType
						.equalsIgnoreCase("SOLO"))) {
			if (null != parameters.get("ccIssueNumber"))
				params.put("CARDISSUE", (String) parameters
						.get("ccIssueNumber"));
			else {
				String creditCardStartYear = (String) parameters
						.get("creditCardStartYear");
				params.put("CARDSTART", (String) parameters
						.get("creditCardStartMonth")
						+ creditCardStartYear.substring(2, 4));
			}
		}

		params.put(PayPalConstants.FIRSTNAME, firstname);
		params.put(PayPalConstants.LASTNAME, lastname);
		params.put(PayPalConstants.COMPANYNAME, companyname);
		params.put(PayPalConstants.STREET, address);
		params.put(PayPalConstants.CITY, city);
		params.put(PayPalConstants.STATE, state);
		params.put(PayPalConstants.ZIP, zip);
		params.put(PayPalConstants.COUNTRY, country);
		params.put(PayPalConstants.EMAIL, email);
		params.put(PayPalConstants.SHIPTOFIRSTNAME, shiptofirstname);
		params.put(PayPalConstants.SHIPTOLASTNAME, shiptolastname);
		params.put(PayPalConstants.SHIPTOSTREET, addressToShip);
		params.put(PayPalConstants.SHIPTOCITY, cityToShip);
		params.put(PayPalConstants.SHIPTOSTATE, stateToShip);
		params.put(PayPalConstants.SHIPTOZIP, zipToShip);
		params.put(PayPalConstants.CUSTREF, (String) parameters.get("custRef"));
		params.put(PayPalConstants.CUSTCODE, (String) parameters
				.get("customerCode"));

		if (null != parameters.get("ipAddress"))
			params.put(PayPalConstants.CLIENTIP, (String) parameters
					.get("ipAddress"));

		params.put(PayPalConstants.CURRENCY, (String) getSettings().getValue(
				"currencyCode"));

		// todo may be we can put ACCTTYPE just for more security. but we r
		// validating pretty good in actionForm

		return params;
	}

	protected void setPaymentVariablesToPersist(Payment payment,
			Map parameters, Map responseMap) {

		String creditCardNumberDisplay = null;
		String decryptedNumber = payment.getDecryptedNumber();

		if (decryptedNumber != null) {
			creditCardNumberDisplay = "XXXXXXXXXXXX"
					+ decryptedNumber.substring(decryptedNumber.length() - 4,
							decryptedNumber.length());
			payment.setCreditCardNumberDisplay(creditCardNumberDisplay);
		}

		payment
				.setValueFieldLabels("AVSADDR,AVSZIP,CVV2MATCH,RESULT, TRXTYPE,AUTHCODE,PNREF,RESPMSG,");
		payment.setValue1((String) responseMap
				.get(PayPalConstants.AVS_ADDR_RESP));
		payment.setValue2((String) responseMap
				.get(PayPalConstants.AVS_ZIP_RESP));
		payment.setValue3((String) responseMap
				.get(PayPalConstants.AVS_CVV2_RESP));
		payment.setValue4((String) responseMap.get(PayPalConstants.RESULT));
		payment.setValue5((String) responseMap.get("transactionType"));
		payment.setValue6((String) responseMap
				.get(PayPalConstants.AUTH_CODE_RESP));
		payment.setValue7((String) responseMap.get(PayPalConstants.PNREF));
		payment.setValue8((String) responseMap.get(PayPalConstants.RESPMSG));
	}

}
