package com.softslate.commerce.businessobjects.product;

import java.util.Collection;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BusinessObject;

/**
 * Interface representing an attribute within the system.
 * <p>
 * When the application needs to create an instance that implements
 * <code>sscAttribute</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "attributeImplementer" setting in the <code>appComponents.properties</code>
 * file.
 * <p>
 * The default "attributeImplementer" is
 * {@link com.softslate.commerce.businessobjects.product.AttributeBean}.
 * 
 * @author David Tobey
 * 
 */
@Audited
@AuditTable("AudAttribute")
public interface Attribute extends BusinessObject {
	abstract int getAttributeID();

	abstract void setAttributeID(int attributeID);

	abstract Double getAttributeAltPrice();

	abstract void setAttributeAltPrice(Double attributeAltPrice);

	abstract String getAttributeCode();

	abstract void setAttributeCode(String attributeCode);

	abstract String getAttributeDescription();

	abstract void setAttributeDescription(String attributeDescription);

	abstract boolean getAttributeIsActive();

	abstract void setAttributeIsActive(boolean attributeIsActive);

	abstract boolean getAttributeIsRequired();

	abstract void setAttributeIsRequired(boolean attributeIsRequired);

	abstract String getAttributeLargeImage();

	abstract void setAttributeLargeImage(String attributeLargeImage);

	abstract String getAttributeName();

	abstract void setAttributeName(String attributeName);

	abstract int getAttributeOrder();

	abstract void setAttributeOrder(int attributeOrder);

	abstract String getAttributeSmallImage();

	abstract void setAttributeSmallImage(String attributeSmallImage);

	abstract String getAttributeType();

	abstract void setAttributeType(String attributeType);

	abstract Double getAttributeUnitCost();

	abstract void setAttributeUnitCost(Double attributeUnitCost);

	abstract Double getAttributeUnitPrice();

	abstract void setAttributeUnitPrice(Double attributeUnitPrice);

	abstract Double getAttributeWeight();

	abstract void setAttributeWeight(Double attributeWeight);

	abstract Integer getDefaultOptionID();

	abstract void setDefaultOptionID(Integer defaultOptionID);

	public Collection getProductAttributes();

	public void setProductAttributes(Collection productAttributes);

	public Collection getAttributeOptions();

	public void setAttributeOptions(Collection attributeOptions);

	public Option getDefaultOption();

	public void setDefaultOption(Option defaultOption);

}
