package com.softslate.commerce.businessobjects.product;

import java.util.Map;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.core.BaseBusinessProcessor;
import com.softslate.commerce.daos.product.ProductGatewayDAO;
import com.softslate.commerce.daos.product.ProductSettingDAO;

/**
 * Processes business logic having to do with product settings. Created in the
 * Struts layer in response to requests for viewing, editing, adding, and
 * deleting product settings.
 * <p>
 * <code>BasicProductSettingProcessor</code> is the default implementation of
 * the <code>ProductSettingProcessor</code> interface for the application.
 * 
 * @author David Tobey
 */
public class BasicProductSettingProcessor extends BaseBusinessProcessor implements ProductSettingProcessor {

	static Log log = LogFactory.getLog(BasicProductSettingProcessor.class);

	public void addProductSetting(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			String code = (String) parameters.get("code");
			log.info(getUser().getAdministrator().getUserName() + ": Adding product setting: " + code);
		}
		ProductSetting productSetting = (ProductSetting) getBusinessObjectFactory().createObject(
				"productSettingImplementer");
		BeanUtils.copyProperties(productSetting, parameters);
		Product product =  getBusinessObjectFactory().createObject(Product.class);
		product.setProductID(productSetting.getProductID());
		productSetting.setProduct(product);
		ProductSettingDAO dao =  getDaoFactory().createDAO(ProductSettingDAO.class);
		dao.setProductSetting(productSetting);
		dao.insertProductSetting(true);
		return;
	}

	public ProductSetting loadProductSettingFromID(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Loading product setting from id");
		ProductSetting productSetting = (ProductSetting) getBusinessObjectFactory().createObject(
				"productSettingImplementer");
		ProductSettingDAO dao =  getDaoFactory().createDAO(ProductSettingDAO.class);
		BeanUtils.copyProperties(productSetting, parameters);
		dao.setProductSetting(productSetting);
		dao.loadProductSettingFromID();
		return productSetting;
	}

	public void editProductSetting(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			String code = (String) parameters.get("code");
			log.info(getUser().getAdministrator().getUserName() + ": Editing product setting: " + code);
		}
		ProductSetting productSetting = (ProductSetting) getBusinessObjectFactory().createObject(
				"productSettingImplementer");
		BeanUtils.copyProperties(productSetting, parameters);
		Product product =  getBusinessObjectFactory().createObject(Product.class);
		product.setProductID(productSetting.getProductID());
		productSetting.setProduct(product);
		ProductSettingDAO dao =  getDaoFactory().createDAO(ProductSettingDAO.class);
		dao.setProductSetting(productSetting);
		dao.updateProductSetting(true);
		return;
	}

	public void deleteProductSetting(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			String code = (String) parameters.get("code");
			log.info(getUser().getAdministrator().getUserName() + ": Deleting product setting: " + code);
		}
		ProductSetting productSetting = (ProductSetting) getBusinessObjectFactory().createObject(
				"productSettingImplementer");
		BeanUtils.copyProperties(productSetting, parameters);
		ProductSettingDAO dao =  getDaoFactory().createDAO(ProductSettingDAO.class);
		dao.setProductSetting(productSetting);
		dao.deleteProductSetting(true);
		return;
	}

	public int loadMaxProductSettingID() throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			log.info(getUser().getAdministrator().getUserName() + ": Loading max product setting ID.");
		}
		ProductGatewayDAO gatewayDAO =  getDaoFactory().createDAO(ProductGatewayDAO.class);
		return gatewayDAO.loadMaxProductSettingID();
	}
}
