package com.softslate.commerce.businessobjects.product;

import java.util.Map;

import com.softslate.commerce.businessobjects.core.BusinessProcessor;

/**
 * Interface for business logic methods having to do with options. Created in
 * the Struts layer in response to requests for viewing, editing, adding, and
 * deleting options.
 * <p>
 * When the application needs to create an instance that implements
 * <code>OptionProcessor</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "optionProcessorImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "optionProcessorImplementer" is
 * {@link com.softslate.commerce.businessobjects.product.OptionProcessor}.
 * 
 * @author David Tobey
 * 
 */
public interface OptionProcessor extends BusinessProcessor {

	abstract Map loadOptionsAndCount(Map parameters) throws Exception;

	abstract Map addOption(Map parameters) throws Exception;

	abstract Option loadOptionFromID(Map parameters) throws Exception;

	abstract Map editOption(Map parameters) throws Exception;

	abstract Map editDeleteOptions(Map parameters) throws Exception;

	abstract Map deleteOption(Map parameters) throws Exception;

	abstract int loadMaxOptionID() throws Exception;

	abstract Option loadOptionFromCode(Map describe) throws Exception;
}
