package com.softslate.commerce.businessobjects.shipping;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BusinessObject;
import com.softslate.commerce.businessobjects.product.SKU;

/**
 * Interface representing a sku shipping rule in the system.
 * <p>
 * When the application needs to create an instance that implements
 * <code>SKUShippingRule</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "skuShippingRuleImplementer" setting in the <code>appComponents.properties</code>
 * file.
 * <p>
 * The default "skuShippingRuleImplementer" is
 * {@link com.softslate.commerce.businessobjects.shipping.SKUShippingRuleBean}.
 * 
 * @author Jason McWilliams
 * 
 */
@Audited
@AuditTable("AudSKUShippingRule")
public interface SKUShippingRule extends BusinessObject {

	abstract int getSkuShippingRuleID();

	abstract void setSkuShippingRuleID(int skuShippingRuleID);

	abstract int getShippingRuleID();

	abstract void setShippingRuleID(int shippingRuleID);

	abstract int getSkuID();

	abstract void setSkuID(int skuID);

	abstract ShippingRule getShippingRule();

	abstract void setShippingRule(ShippingRule discount);

	abstract SKU getSku();

	abstract void setSku(SKU sku);

}
