package com.softslate.commerce.businessobjects.shipping;

import java.io.Serializable;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BaseBusinessObject;

/**
 * Object representing a shipping rate within the system.
 * <code>ShippingRateBean</code> is the default implementation of the
 * <code>ShippingRate</code> interface for the application.
 * <p>
 * The database stores shipping rates in the <code>sscShippingRate</code>
 * table, and this interface can be used to represent a single row of that
 * table. The Hibernate configuration file <code>ShippingRateBean.hbm.xml</code>
 * maps that table's columns to the properties in this object.
 * 
 * @author David Tobey
 * 
 */
@Audited
@AuditTable("AudShippingRate")
public class ShippingRateBean extends BaseBusinessObject implements
		ShippingRate, Serializable {

	private static final long serialVersionUID = -2180354908274017557L;

	private int shippingRateID = 0;

	private int shippingMethodID = 0;

	private Double floor = null;

	private Double ceiling = null;

	private Double amount = null;

	private boolean isActive = true;

	private ShippingMethod shippingMethod = null;

	public ShippingMethod getShippingMethod() {
		return shippingMethod;
	}

	public void setShippingMethod(ShippingMethod shippingMethod) {
		this.shippingMethod = shippingMethod;
	}

	public int getShippingRateID() {
		return shippingRateID;
	}

	public void setShippingRateID(int shippingRateID) {
		this.shippingRateID = shippingRateID;
	}

	public Double getAmount() {
		return amount;
	}

	public void setAmount(Double amount) {
		this.amount = amount;
	}

	public Double getCeiling() {
		return ceiling;
	}

	public void setCeiling(Double ceiling) {
		this.ceiling = ceiling;
	}

	public Double getFloor() {
		return floor;
	}

	public void setFloor(Double floor) {
		this.floor = floor;
	}

	public boolean getIsActive() {
		return isActive;
	}

	public void setIsActive(boolean isActive) {
		this.isActive = isActive;
	}

	public int getShippingMethodID() {
		return shippingMethodID;
	}

	public void setShippingMethodID(int shippingMethodID) {
		this.shippingMethodID = shippingMethodID;
	}
}
