package com.softslate.commerce.customer.order;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.businessobjects.product.Product;
import com.softslate.commerce.businessobjects.product.ProductProcessor;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class that corresponds to the cart item edit page. Used for
 * requests to "/CartItemEditForm.do".
 * <p>
 * The <code>execute</code> method of this class populates a product's
 * information from the database and puts it in the request scope for use by the
 * view layer. The <code>code</code> property of the
 * {@link com.softslate.commerce.businessobjects.order.OrderItem} instance that
 * was populated in <code>CartItemEditFormForm</code> is used to look up the
 * product.
 * 
 * @author David Tobey
 * 
 * @see com.softslate.commerce.customer.order.CartItemEditFormForm
 * @see com.softslate.commerce.businessobjects.product.Product
 * @see com.softslate.commerce.businessobjects.product.ProductProcessor
 */
public class CartItemEditFormAction extends BaseAction {

	static Log log = LogFactory.getLog(CartItemEditFormAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");

		// Get the order item from the form
		CartItemEditFormForm baseForm = (CartItemEditFormForm) form;

		// Load the product from the order item's code property
		ProductProcessor productProcessor = (ProductProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"productProcessorImplementer");
		Product product =  baseForm.getBusinessObjectFactory()
				.createObject(Product.class);
		product.setCode(baseForm.getOrderItem().getCode());
		product = productProcessor.getProductFromCode(product);

		// Error if the product isn't in the database
		if (product.getProductID() == 0) {
			String message = baseForm.getMessage("product", "product.code",
					null);
			throw (new Exception(message + " " + product.getCode()
					+ " not found."));
		}

		// Otherwise copy the product into the request scope.
		request.setAttribute("product", product);
		request.setAttribute("productForm", product);
		request.setAttribute("productSettings", product.getProductSettings());
		request.setAttribute("item", baseForm.getOrderItem());
		request.setAttribute("itemAttributeCollection", baseForm.getOrderItem()
				.getOrderItemAttributes());

		return mapping.findForward("success");
	}

}
