package com.softslate.commerce.customer.order;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionServlet;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.order.OrderProcessor;

/**
 * Initializes singleton objects related to orders in the application scope.
 * Called by {@link com.softslate.commerce.customer.core.BaseRequestProcessor}
 * at the beginning of a request, if the singletons are found to be missing.
 * 
 * @author David Tobey
 * 
 */
public class OrderInitializer {

	static Log log = LogFactory.getLog(OrderInitializer.class);

	public OrderInitializer() {
	}

	public OrderInitializer(ActionServlet servlet) {
		setServlet(servlet);
	}

	private ActionServlet servlet = null;

	public ActionServlet getServlet() {
		return (this.servlet);
	}

	public void setServlet(ActionServlet servlet) {
		this.servlet = servlet;
	}

	/**
	 * Loads the active states and countries for the store into the application
	 * scope. Places an instance of
	 * {@link com.softslate.commerce.businessobjects.order.StatesAndCountries}
	 * in the application scope as an attribute with the name
	 * "statesAndCountries". Invokes
	 * {@link com.softslate.commerce.businessobjects.order.OrderProcessor#loadStatesAndCountries()}
	 * to load the information from the database.
	 */
	public void initializeStatesAndCountries(
			BusinessObjectFactory businessObjectFactory) {
		if (log.isInfoEnabled())
			log.info("Initializing states and countries.");
		try {
			synchronized (getServlet().getServletContext()) {
				OrderProcessor orderProcessor =  businessObjectFactory
						.createObject(OrderProcessor.class);
				getServlet().getServletContext().setAttribute(
						"statesAndCountries",
						orderProcessor.loadStatesAndCountries());
			}
		} catch (Exception e) {
			log.error("Couldn't initialize StatesAndCountries : "+e.toString(), e);
		}
		return;
	}

}
