package com.softslate.commerce.customer.product;

import java.io.PrintWriter;
import java.util.Collection;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.softslate.commerce.businessobjects.product.LuceneSuggestionsSearcher;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Runs a search against the Suggestions Index to provide suggestions for users who are searching.
 */
public class SuggestionsAction extends BaseAction {

	static Log log = LogFactory.getLog(SuggestionsAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form, HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");
		SuggestionsForm baseForm = (SuggestionsForm) form;

		// Reject if Suggestions searching is not on
		String advancedSearchEnableSuggestions = (String) baseForm.getSettings().getValue(
				"advancedSearchEnableSuggestions");
		if (!advancedSearchEnableSuggestions.equals("1")) {
			return mapping.findForward("failure");
		}

		// First create the processor to do the searching
		LuceneSuggestionsSearcher luceneSearcher = baseForm.getBusinessObjectFactory().createObject(
				LuceneSuggestionsSearcher.class);

		// Load the suggestions
		Collection suggestions = luceneSearcher.searchSuggestions(baseForm.getIndexDir(), baseForm.getSearchString());

		// Serialize to json
		response.setContentType("text/json");
		PrintWriter out = response.getWriter();
		Gson gson = new GsonBuilder().disableHtmlEscaping().setPrettyPrinting().create();
		out.println(gson.toJson(suggestions));
		out.flush();
		if (log.isInfoEnabled())
			log.info("Ending action.");
		return null;
	}
}
