package com.softslate.commerce.daos.core;

import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Writer;
import java.security.InvalidKeyException;
import java.security.Key;
import java.security.KeyFactory;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.SecureRandom;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.X509EncodedKeySpec;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import javax.crypto.Cipher;
import javax.crypto.KeyGenerator;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Holds various utility methods for use by DAOs.
 * 
 * @author David Tobey
 */
public class DAOUtilsImpl implements DAOUtils {

	static Log log = LogFactory.getLog(DAOUtilsImpl.class);

	public String[] stopWords = { " a ", " an ", " and ", " for ", " from ",
			" in ", " of ", " on ", " the ", " to ", " with " };

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.softslate.commerce.daos.core.DAOUtilsInt#getStopWords(java.lang.String
	 * [])
	 */
	public String[] getStopWords() {
		return stopWords;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.softslate.commerce.daos.core.DAOUtilsInt#getList(java.lang.String[])
	 */
	public String getList(String[] stringArray) {
		StringBuffer list = new StringBuffer("(");
		for (int i = 0; i < stringArray.length; i++) {
			if (i != 0)
				list.append(", ");
			String nextElement = stringArray[i];
			list.append(nextElement);
		}
		list.append(")");
		return list.toString();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.softslate.commerce.daos.core.DAOUtilsInt#getQuotedList(java.lang.
	 * String[])
	 */
	public String getQuotedList(String[] stringArray) {
		StringBuffer quotedList = new StringBuffer("('");
		for (int i = 0; i < stringArray.length; i++) {
			if (i != 0)
				quotedList.append("', '");
			String nextElement = stringArray[i].replaceAll("'", "");
			quotedList.append(nextElement);
		}
		quotedList.append("')");
		return quotedList.toString();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.softslate.commerce.daos.core.DAOUtilsInt#getListFromResultSet(java
	 * .sql.ResultSet, java.lang.String)
	 */
	public String getListFromResultSet(ResultSet resultSet, String field)
			throws SQLException {
		StringBuffer list = new StringBuffer("(");
		int count = 0;
		while (resultSet.next()) {
			if (count != 0)
				list.append(", ");
			int nextElement = resultSet.getInt(field);
			list.append(Integer.toString(nextElement));
			count++;
		}
		list.append(")");
		return list.toString();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.softslate.commerce.daos.core.DAOUtilsInt#getListFromCollection(java
	 * .util.Collection, java.lang.String)
	 */
	public String getListFromCollection(Collection collection, String field) {
		StringBuffer list = new StringBuffer("(");
		Iterator i = collection.iterator();
		int count = 0;
		while (i.hasNext()) {
			if (count != 0)
				list.append(", ");
			Map nextItem = (Map) i.next();
			String item = (String) nextItem.get(field);
			list.append(item);
			count++;
		}
		list.append(")");
		return list.toString();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.softslate.commerce.daos.core.DAOUtilsInt#getListFromStringCollection
	 * (java.util.Collection)
	 */
	public String getListFromStringCollection(Collection collection) {
		StringBuffer list = new StringBuffer("(");
		Iterator i = collection.iterator();
		int count = 0;
		while (i.hasNext()) {
			if (count != 0)
				list.append(", ");
			String item = (String) i.next();
			list.append(item);
			count++;
		}
		list.append(")");
		return list.toString();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.softslate.commerce.daos.core.DAOUtilsInt#getCollectionFromResultSet
	 * (java.sql.ResultSet, java.lang.String)
	 */
	public Collection getCollectionFromResultSet(ResultSet resultSet,
			String field) throws SQLException {
		Collection list = new ArrayList();
		while (resultSet.next()) {
			int nextElement = resultSet.getInt(field);
			list.add(Integer.toString(nextElement));
		}
		return list;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.softslate.commerce.daos.core.DAOUtilsInt#getCollectionFromCollection
	 * (java.util.Collection, java.lang.String)
	 */
	public Collection getCollectionFromCollection(Collection collection,
			String field) {
		Collection list = new ArrayList();
		Iterator i = collection.iterator();
		while (i.hasNext()) {
			Map nextItem = (Map) i.next();
			list.add(nextItem.get(field));
		}
		return list;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.softslate.commerce.daos.core.DAOUtilsInt#getIntegersFromStrings(java
	 * .util.Collection)
	 */
	public Collection getIntegersFromStrings(Collection collection) {
		Collection list = new ArrayList();
		Iterator i = collection.iterator();
		while (i.hasNext()) {
			String nextItem = (String) i.next();
			list.add(new Integer(nextItem));
		}
		return list;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.softslate.commerce.daos.core.DAOUtilsInt#getCollectionFromBeans(java
	 * .util.Collection, java.lang.String)
	 */
	public Collection getCollectionFromBeans(Collection collection, String field)
			throws Exception {
		Collection list = new ArrayList();
		Iterator i = collection.iterator();
		while (i.hasNext()) {
			Object nextItem = (Object) i.next();
			if (nextItem instanceof Integer) {
				list.add(nextItem);
			} else {
				String value = BeanUtils.getProperty(nextItem, field);
				Integer integer = new Integer(0);
				if (value != null) {
					integer = new Integer(Integer.parseInt(value));
				}
				list.add(integer);
			}
		}
		return list;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.softslate.commerce.daos.core.DAOUtilsInt#getDebugMessage(java.sql
	 * .ResultSet)
	 */
	public String getDebugMessage(ResultSet resultSet) throws SQLException {
		resultSet.last();
		String message = "Query results: " + resultSet.getRow()
				+ " row(s) and "
				+ Integer.toString(resultSet.getMetaData().getColumnCount())
				+ " column(s).";
		resultSet.beforeFirst();
		return message;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.softslate.commerce.daos.core.DAOUtilsInt#getQuotedList(java.util.Map)
	 */
	public String getQuotedList(Map map) {
		StringBuffer quotedList = new StringBuffer("('");
		Set keySet = map.keySet();
		Iterator i = keySet.iterator();
		int count = 0;
		while (i.hasNext()) {
			String key = (String) i.next();
			if (count != 0) {
				quotedList.append("', '");
			}
			String nextElement = key.replaceAll("'", "");
			quotedList.append(nextElement);
			count++;
		}
		quotedList.append("')");
		return quotedList.toString();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.softslate.commerce.daos.core.DAOUtilsInt#outputCsv(java.util.Collection
	 * , java.io.Writer)
	 */
	public void outputCsv(Collection data, Writer out) throws IOException {
		Iterator it = data.iterator();
		while (it.hasNext()) {
			Object[] row = (Object[]) it.next();
			if (row != null) {
				for (int ix = 0; ix < row.length; ix++) {
					String s = "";
					if (row[ix] != null)
						s = row[ix].toString();
					if (ix != 0)
						out.write(',');
					if (s.indexOf(',') == -1 && s.indexOf('"') == -1
							&& s.indexOf('\n') == -1)
						out.write(s);
					else {
						out.write('"');
						int length = s.length();
						for (int i = 0; i < length; i++) {
							char c = s.charAt(i);
							if (c == '"' || c == '\\') {
								out.write('\\');
								out.write(c);
							} else if (c == '\n')
								out.write("\\n");
							else
								out.write(c);
						}
						out.write('"');
					}
				}
			}
			out.write("\n");
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.softslate.commerce.daos.core.DAOUtilsInt#prepareSearchString(java
	 * .lang.String)
	 */
	public String prepareSearchString(String searchString) {
		// Take out characters dangerous to a LIKE SQL expression.
		searchString = searchString.replaceAll("[%\\[\\]\\^]", "");
		// Replace stop words with wildcards to help find phrases.
		for (int i = 0; i < stopWords.length; i++) {
			searchString = searchString.replaceAll(stopWords[i], "%");
		}
		// Replace spaces with wildcards, to help search for phrases.
		searchString = searchString.replace(' ', '%');
		// Make the string lower case for case insensitivity
		searchString = searchString.toLowerCase();
		searchString = "%" + searchString + "%";
		return searchString;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.softslate.commerce.daos.core.DAOUtilsInt#getEncrypted(java.lang.String
	 * , java.lang.String, java.lang.String)
	 */
	public String getEncrypted(String stringToEncrypt, String encryptionType,
			String publicKeyFile) {
		return this.getEncrypted(stringToEncrypt, encryptionType, publicKeyFile, "md5");
	}

	public String getEncrypted(String stringToEncrypt, String encryptionType, String publicKeyFile,
			String oneWayAlgorithm) {
		try {
			if (log.isDebugEnabled()) {
				// log.debug("stringToEncrypt: " + stringToEncrypt);
				log.debug("encryptionType: " + encryptionType);
				log.debug("publicKeyFile: " + publicKeyFile);
			}
			if (stringToEncrypt == null || stringToEncrypt.length() == 0) {
				return stringToEncrypt;
			}
			if (encryptionType == null) {
				return stringToEncrypt;
			}
			if (encryptionType.equals("twoWay")) {
				ObjectInputStream in = new ObjectInputStream(
						new FileInputStream(publicKeyFile));
				Object object = in.readObject();
				Key key = (javax.crypto.SecretKey) object;
				in.close();
				Cipher cipher = Cipher.getInstance("DES/ECB/PKCS5Padding");
				try {
					cipher.init(Cipher.ENCRYPT_MODE, key);
				} catch (InvalidKeyException ike) {
					cipher = Cipher.getInstance("AES");
					cipher.init(Cipher.ENCRYPT_MODE, key);
				}
				byte[] inputBytes = stringToEncrypt.getBytes("UTF8");
				byte[] outputBytes = cipher.doFinal(inputBytes);
				byte[] encoded = Base64.encodeBase64(outputBytes);
				String encryptedString = new String(encoded, "UTF8");
				if (log.isDebugEnabled())
					log.debug("encryptedString: " + encryptedString);
				return encryptedString;
			} else if (encryptionType.equals("keyPair")) {
				Cipher cipher = Cipher.getInstance("RSA/ECB/OAEPPadding");
				PublicKey key = (PublicKey) loadKey(publicKeyFile, true);
				cipher.init(Cipher.ENCRYPT_MODE, key);
				byte[] encryptedBytes = cipher.doFinal(stringToEncrypt
						.getBytes("UTF8"));
				byte[] encoded = Base64.encodeBase64(encryptedBytes);
				String encryptedString = new String(encoded, "UTF8");
				if (log.isDebugEnabled())
					log.debug("encryptedString: " + encryptedString);
				return encryptedString;
			} else if (encryptionType.equals("oneWay") && (oneWayAlgorithm == null || oneWayAlgorithm.equals("md5"))) {
				if (log.isDebugEnabled())
					log.debug("DigestUtils.md5Hex(stringToEncrypt): "
							+ DigestUtils.md5Hex(stringToEncrypt));
				return DigestUtils.md5Hex(stringToEncrypt);
			} else if (encryptionType.equals("oneWay") && oneWayAlgorithm.equals("sha256")) {
				if (log.isDebugEnabled())
					log.debug("DigestUtils.sha256Hex(stringToEncrypt): " + DigestUtils.sha256Hex(stringToEncrypt));
				return DigestUtils.sha256Hex(stringToEncrypt);
			} else if (encryptionType.equals("notStored")) {
				if (log.isDebugEnabled())
					log.debug("Encryption type: not stored.");
				return "";
			} else {
				return stringToEncrypt;
			}
		} catch (Exception e) {
			log.error("Error encrypting string: " + e.toString(), e);
			return stringToEncrypt;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.softslate.commerce.daos.core.DAOUtilsInt#getDecrypted(java.lang.String
	 * , java.lang.String, java.lang.String)
	 */
	public String getDecrypted(String stringToDecrypt, String encryptionType,
			String privateKeyFile) {
		try {
			if (log.isDebugEnabled()) {
				log.debug("stringToDecrypt: " + stringToDecrypt);
				log.debug("encryptionType: " + encryptionType);
				log.debug("privateKeyFile: " + privateKeyFile);
			}
			if (stringToDecrypt == null || stringToDecrypt.length() == 0) {
				return stringToDecrypt;
			}
			if (encryptionType == null) {
				return stringToDecrypt;
			}
			if (encryptionType.equals("twoWay")) {
				ObjectInputStream in = new ObjectInputStream(
						new FileInputStream(privateKeyFile));
				Object object = in.readObject();
				Key key = (javax.crypto.SecretKey) object;
				in.close();
				Cipher cipher = Cipher.getInstance("DES/ECB/PKCS5Padding");
				try {
					cipher.init(Cipher.DECRYPT_MODE, key);
				} catch (InvalidKeyException ike) {
					cipher = Cipher.getInstance("AES");
					cipher.init(Cipher.DECRYPT_MODE, key);
				}
				byte[] encryptedBytes = stringToDecrypt.getBytes("UTF8");
				byte[] inputBytes = Base64.decodeBase64(encryptedBytes);
				byte[] outputBytes = cipher.doFinal(inputBytes);
				String decryptedString = new String(outputBytes, "UTF8");
				// if (log.isDebugEnabled())
				// log.debug("decryptedString: " + decryptedString);
				return decryptedString;
			} else if (encryptionType.equals("keyPair")) {
				Cipher cipher = Cipher.getInstance("RSA/ECB/OAEPPadding");
				PrivateKey key = (PrivateKey) loadKey(privateKeyFile, false);
				cipher.init(Cipher.DECRYPT_MODE, key);
				byte[] encryptedBytes = stringToDecrypt.getBytes("UTF8");
				byte[] bytesToDecrypt = Base64.decodeBase64(encryptedBytes);
				byte[] decryptedBytes = cipher.doFinal(bytesToDecrypt);
				String decryptedString = new String(decryptedBytes, "UTF8");
				// if (log.isDebugEnabled())
				// log.debug("decryptedString: " + decryptedString);
				return decryptedString;
			} else {
				return stringToDecrypt;
			}
		} catch (Exception e) {
			log.error("Error decrypting string: " + e.toString(), e);
			return stringToDecrypt;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.softslate.commerce.daos.core.DAOUtilsInt#generateTwoWayKey(java.lang
	 * .String)
	 */
	public void generateTwoWayKey(String keyFile) throws Exception {
		if (log.isDebugEnabled())
			log.debug("Attempting to create key file");
		if (log.isDebugEnabled())
			log.debug("keyFile: " + keyFile);
		KeyGenerator generator = KeyGenerator.getInstance("AES");
		generator.init(new SecureRandom());
		Key key = generator.generateKey();
		ObjectOutputStream out = new ObjectOutputStream(new FileOutputStream(
				keyFile));
		out.writeObject(key);
		out.close();
		return;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.softslate.commerce.daos.core.DAOUtilsInt#generateKeyPair(java.lang
	 * .String, java.lang.String, boolean)
	 */
	public String generateKeyPair(String publicFile, String privateFile,
			boolean savePrivate) throws Exception {
		if (log.isDebugEnabled())
			log.debug("Attempting to create key files");
		if (log.isDebugEnabled())
			log.debug("publicFile: " + publicFile);
		if (log.isDebugEnabled())
			log.debug("privateFile: " + privateFile);
		KeyPairGenerator kpg = KeyPairGenerator.getInstance("RSA", "BC");
		kpg.initialize(1024, new SecureRandom());
		KeyPair kp = kpg.generateKeyPair();
		PrivateKey priKey = kp.getPrivate();
		if (savePrivate) {
			saveKey(privateFile, priKey.getEncoded());
		}
		PublicKey pubKey = kp.getPublic();
		saveKey(publicFile, pubKey.getEncoded());
		byte[] encoded = Base64.encodeBase64(priKey.getEncoded());
		return new String(encoded, "UTF8");
	}

	private void saveKey(String filename, byte[] contents) throws Exception {
		FileOutputStream fos = new FileOutputStream(filename);
		fos.write(contents);
		fos.flush();
		fos.close();
	}

	private Key loadKey(String filename, boolean isPublic) throws Exception {
		FileInputStream fis = null;
		fis = new FileInputStream(filename);
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		int b;
		while ((b = fis.read()) != -1) {
			baos.write(b);
		}
		byte[] keyBytes = baos.toByteArray();
		fis.close();
		Key key = null;
		KeyFactory kf = KeyFactory.getInstance("RSA", "BC");
		if (isPublic) {
			X509EncodedKeySpec publicKey = new X509EncodedKeySpec(keyBytes);
			key = kf.generatePublic(publicKey);
			return key;
		} else {
			PKCS8EncodedKeySpec privateKey = new PKCS8EncodedKeySpec(keyBytes);
			key = kf.generatePrivate(privateKey);
			return key;
		}
	}

}
