package com.softslate.commerce.daos.order;

import java.util.Calendar;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.Hibernate;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;

import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.businessobjects.order.OrderDelivery;
import com.softslate.commerce.businessobjects.order.OrderItem;
import com.softslate.commerce.businessobjects.order.OrderSetting;
import com.softslate.commerce.daos.core.BaseDAO;
import com.softslate.commerce.daos.core.DataAccessException;
import com.softslate.commerce.daos.order.OrderSettingDAO;

/**
 * Generic DAO to manage the <code>sscOrder</code> table of the database.
 * 
 * @author David Tobey
 * 
 */
public class OrderDAOHibernate extends BaseDAO implements OrderDAO {

	static Log log = LogFactory.getLog(OrderDAOHibernate.class);

	private Order order = null;

	public Order getOrder() {
		return (this.order);
	}

	public void setOrder(Order order) {
		this.order = order;
	}

	public Order loadOrderFromID(int orderID) throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			Order record = (Order) session.get(Order.class, new Integer(
					orderID));
			return record;
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

	public void loadOrderFromID() throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			Order record = (Order) session.get(Order.class, new Integer(
					getOrder().getOrderID()));
			setOrder(record);
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void loadOrderFromNumber() throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "order.selectFromNumber");
			q.setInteger("orderNumber", getOrder().getOrderNumber().intValue());
			Order order = (Order) q.uniqueResult();
			setOrder(order);
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public Object loadObject(Object object) throws Exception {
		if (null != object && object instanceof Order) {
			this.setOrder((Order) object);
			if (0 != this.getOrder().getOrderID()) {
				this.loadOrderFromID();
			} else if (null != this.getOrder().getOrderNumber()) {
				this.loadOrderFromNumber();
			}
			return this.getOrder();
		}
		return null;
	}

	public int getMaxOrderNumber() throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "order.maxOrderNumber");
			Integer maxNum = (Integer) q.uniqueResult();
			if (maxNum == null) {
				return 0;
			}
			return maxNum.intValue();
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

	public void insertOrder() throws DataAccessException {
		insertOrder(false);
	}

	public void insertOrder(boolean commit) throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			getOrder().setCreated(formatDateTime(new java.util.Date()));
			getOrder().setLastModified(formatDateTime(new java.util.Date()));
			session.save(getOrder());
			if (commit) {
				getDaoFactory().commitTransaction();
			}
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public Object insertObject(Object object, boolean commit) throws Exception {
		if (null != object && object instanceof Order) {
			this.setOrder((Order) object);
			this.insertOrder(commit);
			return this.getOrder();
		}
		return null;
	}

	public void updateOrder() throws Exception {
		updateOrder(false);
	}

	public void updateOrder(boolean commit) throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			getOrder().setLastModified(formatDateTime(new java.util.Date()));
			session.update(getOrder());
			if (commit) {
				getDaoFactory().commitTransaction();
			}
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public Object updateObject(Object object, boolean commit) throws Exception {
		if (null != object && object instanceof Order) {
			this.setOrder((Order) object);
			this.updateOrder(commit);
			return this.getOrder();
		}
		return null;

	}

	public void loadFromNumberAndCustomerID() throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session,
					"order.selectFromNumberAndCustomerID");
			q.setInteger("orderNumber", getOrder().getOrderNumber().intValue());
			q.setInteger("customerID", getOrder().getCustomerID().intValue());
			Order order = (Order) q.uniqueResult();
			setOrder(order);
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void loadSavedCart() throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			// Get the number of days carts can be saved
			String savedCartDays = (String) getSettings().getValue(
					"savedCartDays");
			int daysAgo = 0;
			if (savedCartDays == null || savedCartDays.equals("0")) {
				daysAgo = -1000000;
			} else {
				daysAgo = -1 * Integer.parseInt(savedCartDays);
			}
			Calendar start = Calendar.getInstance();
			start.add(Calendar.DATE, daysAgo);
			// Construct the query
			Query q = getNamedQuery(session, "order.selectSavedCart");
			q.setInteger("orderID", getOrder().getOrderID());
			q.setInteger("customerID", getOrder().getCustomerID().intValue());
			q.setString("daysAgo", formatDateTime(new java.util.Date(start
					.getTimeInMillis())));
			Order order = (Order) q.uniqueResult();
			// Initialize the entire tree so it can be used across Hibernate
			// sessions
			initializeOrder(order);
			setOrder(order);
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void loadSavedCartFromToken() throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			// Get the number of days carts can be saved
			String savedCartDays = (String) getSettings().getValue(
					"savedCartDays");
			int daysAgo = 0;
			if (savedCartDays == null || savedCartDays.equals("0")) {
				daysAgo = -1000000;
			} else {
				daysAgo = -1 * Integer.parseInt(savedCartDays);
			}
			Calendar start = Calendar.getInstance();
			start.add(Calendar.DATE, daysAgo);
			Query q = getNamedQuery(session, "order.selectFromSavedCartToken");
			q.setString("autoSavedCartToken", getOrder()
					.getAutoSavedCartToken());
			q.setString("daysAgo", formatDateTime(new java.util.Date(start
					.getTimeInMillis())));
			// Only get the most recent order matching the token
			Collection orders = q.list();
			Order order = null;
			Iterator i = orders.iterator();
			if (i.hasNext()) {
				order = (Order) i.next();
				// Initialize the entire tree so it can be used across Hibernate
				// sessions
				initializeOrder(order);
			}
			setOrder(order);
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void loadPreviousOrder() throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			// Get the number of days previous orders can be picked up
			String reorderDays = (String) getSettings().getValue("reorderDays");
			int daysAgo = 0;
			if (reorderDays == null || reorderDays.equals("0")) {
				daysAgo = -1000000;
			} else {
				daysAgo = -1 * Integer.parseInt(reorderDays);
			}
			Calendar start = Calendar.getInstance();
			start.add(Calendar.DATE, daysAgo);
			// Construct the query
			Query q = getNamedQuery(session, "order.selectPreviousOrder");
			q.setInteger("orderID", getOrder().getOrderID());
			q.setInteger("customerID", getOrder().getCustomerID().intValue());
			q.setString("daysAgo", formatDateTime(new java.util.Date(start
					.getTimeInMillis())));
			Order order = (Order) q.uniqueResult();
			// Initialize the entire tree so it can be used across Hibernate
			// sessions
			initializeOrder(order);
			setOrder(order);
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void deleteOrder() throws DataAccessException {
		deleteOrder(false);
	}

	public void deleteOrder(boolean commit) throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			setOrder((Order) session.load(Order.class, new Integer(getOrder()
					.getOrderID())));
			session.delete(getOrder());
			if (commit) {
				getDaoFactory().commitTransaction();
			}
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void initializeOrder(Order order) {
		if (order != null) {
			Hibernate.initialize(order.getOrderDeliveryCollection());
			Hibernate.initialize(order.getOrderDiscounts());
			Hibernate.initialize(order.getPayments());
			if (order.getOrderDeliveryCollection() != null) {
				Iterator i = order.getOrderDeliveryCollection().iterator();
				while (i.hasNext()) {
					OrderDelivery od = (OrderDelivery) i.next();
					Hibernate.initialize(od.getOrderItemCollection());
					if (od.getOrderItemCollection() != null) {
						Iterator j = od.getOrderItemCollection().iterator();
						while (j.hasNext()) {
							OrderItem oi = (OrderItem) j.next();
							Hibernate.initialize(oi.getOrderItemAttributes());
							Hibernate.initialize(oi.getOrderDiscounts());
						}
					}
				}
			}
		}
	}

	public Collection loadSettings() throws DataAccessException {
		Session session = getDaoFactory().startSession();
		Collection results = null;
		try {
			Query q = getNamedQuery(session, "orderSetting.selectAll");
			q.setInteger("orderID", getOrder().getOrderID());
			results = new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return results;
	}

	public Collection processSettings(Map parameters) throws Exception {
		return processSettings(false, parameters);
	}

	public Collection processSettings(boolean commit, Map parameters)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Processing order settings.");
		OrderSettingDAO orderSettingDAO = (OrderSettingDAO) getDaoFactory()
				.createDAO("orderSettingDAOImplementer");
		OrderDAO orderDAO = (OrderDAO) parameters.get("orderDAO");
		
		// Update all the existing settings
		if (log.isDebugEnabled())
			log.debug("Processing order setting updates.");
		Collection settings = (Collection) parameters.get("settings");
		String[] codes = (String[]) parameters.get("code");
		String[] values = (String[]) parameters.get("value");
		String[] valueTypes = (String[]) parameters.get("valueType");
		for (int i = 0; i < codes.length; i++) {
			Iterator j = settings.iterator();
			while (j.hasNext()) {
				OrderSetting setting = (OrderSetting) j.next();
				if (setting.getCode().equals(codes[i])) {
					if (log.isDebugEnabled())
						log.debug("Existing order setting: " + codes[i]);
					if (valueTypes[i].equals("integerValue")
							|| valueTypes[i].equals("booleanValue")) {
						setting.setIntegerValue(Integer.parseInt(values[i]));
					} else if (valueTypes[i].equals("smallValue")) {
						setting.setSmallValue(values[i]);
					} else if (valueTypes[i].equals("mediumValue")) {
						setting.setMediumValue(values[i]);
					} else if (valueTypes[i].equals("textValue")) {
						setting.setTextValue(values[i]);
					}
				}
			}
		}
		
		// Insert the new settings
		if (log.isDebugEnabled())
			log.debug("Processing order setting inserts.");
		Collection orderSettingsToInsert = (Collection) parameters
				.get("orderSettingsToInsert");
		Iterator i = orderSettingsToInsert.iterator();
		while (i.hasNext()) {
			orderSettingDAO.setOrderSetting((OrderSetting) i.next());
			orderSettingDAO.insertOrderSetting();
		}
		
		// Reload the settings and return them
		Collection newSettings = orderDAO.loadSettings();
		if (commit) {
			getDaoFactory().commitTransaction();
		}
		return newSettings;
	}

}
