package com.softslate.commerce.daos.shipping;

import java.util.Collection;
import java.util.LinkedHashSet;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.type.IntegerType;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.daos.core.BaseAdminGatewayDAOHibernate;
import com.softslate.commerce.daos.core.DataAccessException;

/**
 * Generic DAO that runs miscellaneous queries related to shipping rules.
 * 
 * @author Jason McWilliams
 */
public class ShippingRuleGatewayDAOHibernate extends BaseAdminGatewayDAOHibernate implements ShippingRuleGatewayDAO {

	static Log log = LogFactory.getLog(ShippingRuleGatewayDAOHibernate.class);

	public Map loadShippingRulesAndCount(Map parameters) throws Exception {
		return loadItemsAndCount(parameters);
	}

	public Map editDeleteShippingRules(Map parameters, BusinessObjectFactory bof) throws Exception {
		Map result = editDeleteItems(parameters, bof);
		evictCache();
		return result;
	}

	public Collection getShippingRulesBySkuIDs(Collection skuIDs) throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "shippingRule.shippingPackageRulesBySkuIDs");
			q.setParameterList("skuIDs", skuIDs, IntegerType.INSTANCE);
			q.setCacheable(true);
			return new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

	public Collection loadLimitByLocationShippingRules() throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "shippingRule.selectLimitByLocationRules");
			q.setCacheable(true);
			return new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

}
